@@ -11,7 +11,7 @@
 #include "util/arg_auto_set.h"
 #include "Rev.h"
 
-#define version "3.2.2"
+#define version "4.0.2"
 
 #define LIBRND_ROOT         "../src/librnd"
 #define LIBRND_PLUGIN_ROOT  "../src/librnd/plugins"
@@ -95,15 +95,12 @@ int hook_custom_arg(const char *key, const char *value)
 		return 1;
 	}
 	if (strcmp(key, "disable-so") == 0) {
-		put("/local/pcb/disable_so", strue);
 		put("/local/librnd/disable_so", strue);
-		put("/local/pcb/want_static_librnd", strue);
 		put("/local/librnd/want_static_librnd", strue);
 		pup_set_debug(strue);
 		return 1;
 	}
 	if (strcmp(key, "static-librnd") == 0) {
-		put("/local/pcb/want_static_librnd", strue);
 		put("/local/librnd/want_static_librnd", strue);
 		pup_set_debug(strue);
 		return 1;
@@ -318,14 +315,14 @@ static void rnd_hook_detect_coord_bits(void)
 	int long_bits      = safe_atoi(get("sys/types/size/signed_long_int")) * 8;
 	int long_long_bits = safe_atoi(get("sys/types/size/signed_long_long_int")) * 8;
 	int int64_bits     = safe_atoi(get("sys/types/size/uint64_t")) * 8;
-	const char *chosen, *postfix;
+	const char *chosen, *uchosen, *postfix;
 	char tmp[64];
 	int need_stdint = 0;
 
-	if (want_coord_bits == int_bits)             { postfix="U";   chosen = "int";           }
-	else if (want_coord_bits == long_bits)       { postfix="UL";  chosen = "long int";      }
-	else if (want_coord_bits == int64_bits)      { postfix="ULL"; chosen = "int64_t";       need_stdint = 1; }
-	else if (want_coord_bits == long_long_bits)  { postfix="ULL"; chosen = "long long int"; }
+	if (want_coord_bits == int_bits)             { postfix="U";   chosen = "int";           uchosen = "unsigned int";          }
+	else if (want_coord_bits == long_bits)       { postfix="UL";  chosen = "long int";      uchosen = "unsigned long int";     }
+	else if (want_coord_bits == int64_bits)      { postfix="ULL"; chosen = "int64_t";       uchosen = "unsigned int64_t";      need_stdint = 1; }
+	else if (want_coord_bits == long_long_bits)  { postfix="ULL"; chosen = "long long int"; uchosen = "unsigned long long int";}
 	else {
 		report("ERROR: can't find a suitable integer type for coord to be %d bits wide\n", want_coord_bits);
 		exit(1);
@@ -334,6 +331,7 @@ static void rnd_hook_detect_coord_bits(void)
 	sprintf(tmp, "((1%s<<%d)-1)", postfix, want_coord_bits - 1);
 	put("/local/pcb/coord_type", chosen);
 	put("/local/librnd/coord_type", chosen);
+	put("/local/librnd/ucoord_type", uchosen);
 	put("/local/pcb/coord_max", tmp);
 	put("/local/librnd/coord_max", tmp);
 
@@ -364,11 +362,11 @@ static void rnd_hook_detect_coord_bits(void)
    a pup dep because it is not in librnd but in the host app and can have a
    different na,e */
 static const char *dialog_deps[] = {
-	"/local/pcb/dialogs/controls",         /* so we don't relax user's explicit request */
-	"/local/pcb/hid_remote/controls",
-	"/local/pcb/lib_gtk2_common/controls",
-	"/local/pcb/lib_gtk4_common/controls",
-	"/local/pcb/hid_lesstif/controls",
+	"/local/module/dialogs/controls",         /* so we don't relax user's explicit request */
+	"/local/module/hid_remote/controls",
+	"/local/module/lib_gtk2_common/controls",
+	"/local/module/lib_gtk4_common/controls",
+	"/local/module/hid_lesstif/controls",
 	NULL
 };
 
@@ -482,6 +480,8 @@ static int rnd_hook_generate()
 	printf("Generating librnd config.h (%d)\n", generr |= tmpasm(LIBRND_ROOT, "config.h.in", "config.h"));
 	printf("Generating opengl.h (%d)\n", generr |= tmpasm(LIBRND_PLUGIN_ROOT "/lib_hid_gl", "opengl.h.in", "opengl.h"));
 	printf("Generating draw_INIT.h (%d)\n", generr |= tmpasm(LIBRND_PLUGIN_ROOT "/lib_hid_gl", "draw_INIT.h.in", "draw_INIT.h"));
+	printf("Generating poly conf h (%d)\n", generr |= tmpasm(LIBRND_ROOT "/poly", "polyconf.h.in", "polyconf.h"));
+	printf("Generating polybool conf h (%d)\n", generr |= tmpasm(LIBRND_ROOT "/polybool", "polyconf.h.in", "polyconf.h"));
 
 	return generr;
 }
@@ -551,7 +551,7 @@ int hook_generate()
 	printf("=====================\n");
 
 	print_sum_setting("/local/librnd/debug",          "Compilation for debugging");
-	print_sum_setting_or("/local/librnd/symbols",     "Include debug symbols", istrue(get("/local/pcb/debug")));
+	print_sum_setting_or("/local/librnd/symbols",     "Include debug symbols", istrue(get("/local/rnd/debug")));
 	print_sum_cfg_val("/local/librnd/coord_bits",     "Coordinate type bits");
 
 #undef plugin_def
@@ -569,7 +569,7 @@ int hook_generate()
 #undef plugin_def
 #undef plugin_header
 #undef plugin_dep
-#define plugin_def(name, desc, default_, all_) fprintf(f, "#state %s %s\n", name, get("/local/pcb/" name "/controls"));
+#define plugin_def(name, desc, default_, all_) fprintf(f, "#state %s %s\n", name, get("/local/module/" name "/controls"));
 #define plugin_header(sect)
 #define plugin_dep(plg, on)
 #include "plugins.h"
@@ -577,7 +577,7 @@ int hook_generate()
 #undef plugin_def
 #undef plugin_header
 #undef plugin_dep
-#define plugin_def(name, desc, default_, all_) fprintf(f, "/local/pcb/%s/controls=%s\n/local/pcb/%s/external=true\n", name, get("/local/pcb/" name "/controls"), name);
+#define plugin_def(name, desc, default_, all_) fprintf(f, "/local/module/%s/controls=%s\n/local/module/%s/external=true\n", name, get("/local/module/" name "/controls"), name);
 #define plugin_header(sect)
 #define plugin_dep(plg, on)
 #include "plugins.h"
