@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -94,6 +94,16 @@ typedef struct {
 	lht_node_t *src;
 } rnd_confprop_t;
 
+/* Shared part of rnd_conf_native_t: these are allocated only once, as an
+   over-allocation in rnd_conf_fields_master and per design conf natives
+   are just pointing to it. Stores hid/plugin registrations that affect all
+   designs */
+typedef struct rnd_conf_nat_shared_s {
+	vtp0_t hid_data;
+	vtp0_t hid_callbacks; /* vector of (const rnd_conf_hid_callbacks_t *); shared among all design instances */
+	const char *gui_edit_act; /* if non-zero, run this action for GUI editing; args: role_name, full_list_conf_path, item_name */
+} rnd_conf_nat_shared_t;
+
 struct rnd_conf_native_s {
 	/* static fields defined by the macros */
 	const char *description;
@@ -116,10 +126,9 @@ struct rnd_conf_native_s {
 	int rnd_conf_rev;     /* last changed rev */
 
 	/* dynamic fields for HIDs storing their data */
-	vtp0_t hid_data;
-	vtp0_t hid_callbacks; /* vector of (const rnd_conf_hid_callbacks_t *) */
+	rnd_conf_nat_shared_t *shared;
 
-	const char *gui_edit_act; /* if non-zero, run this action for GUI editing; args: role_name, full_list_conf_path, item_name */
+	void *shared_static; /* for over-allocation for ->shared in rnd_conf_fields_master */
 };
 
 struct conf_listitem_s {
@@ -168,7 +177,7 @@ void rnd_conf_load_all(const char *project_fn, const char *design_fn);
 extern int rnd_conf_in_production;
 
 /* Load a file or a string as a role
-   [3.2.0]: if fn is NULL, create an empty lihata document for the role */
+   if fn is NULL, create an empty lihata document for the role */
 int rnd_conf_load_as(rnd_conf_role_t role, const char *fn, int fn_is_text);
 
 /* copy root to be the new config of role; root must be a li:APPNAME-conf-v1
@@ -253,8 +262,8 @@ const char *rnd_conf_role_name(rnd_conf_role_t r);
    fields may be modified but the structure of the tree is static (can't
    create or remove nodes). This is useful when an io_ file format supports
    only a subset of settings: it can build the RND_CFR_DESIGN tree, lock it so
-   settings that it wouldn't know how to save won't appear. NOTE: io_pcb
-   supports all settings via attributes so does not lock. */
+   settings that it wouldn't know how to save won't appear. NOTE: pcb-rnd's 
+   io_pcb supports all settings via attributes so does not lock. */
 void rnd_conf_lock(rnd_conf_role_t target);
 void rnd_conf_unlock(rnd_conf_role_t target);
 
@@ -266,8 +275,12 @@ int rnd_conf_replace_subtree(rnd_conf_role_t dst_role, const char *dst_path, rnd
    Useful for io_ plugins, on RND_CFR_DESIGN, before loading a new file. */
 void rnd_conf_reset(rnd_conf_role_t target, const char *source_fn);
 
+/* Free the lht doc for the given role */
+void rnd_conf_free(rnd_conf_role_t target);
+
+
 /* Save an in-memory lihata representation to the disk */
-int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char *design_fn, rnd_conf_role_t role, const char *fn);
+int rnd_conf_save_file(rnd_design_t *design, const char *project_fn, const char *design_fn, rnd_conf_role_t role, const char *fn);
 
 /* Returns whether a given lihata tree is locked */
 int rnd_conf_islocked(rnd_conf_role_t target);
@@ -305,23 +318,36 @@ void rnd_conf_ro(const char *path);
 
 void rnd_conf_setf(rnd_conf_role_t role, const char *path, int idx, const char *fmt, ...);
 
-#define rnd_conf_list_foreach_path_first(hidlib, res, conf_list, call) \
+/* execute call() on each string of a conflist, saving result in res
+   and break the first time res == 0. Saves a list of strings to run on
+   so changing the list or calling conf_merge() is safe to call within call()
+   and it's always the "list at the time of the call" that is used. */
+#define rnd_conf_list_foreach_path_first(design, res, conf_list, call) \
 do { \
 	rnd_conf_listitem_t *__n__; \
 	const rnd_conflist_t *__lst1__ = (conf_list); \
 	rnd_conflist_t *__lst__ = (rnd_conflist_t *)(__lst1__); \
 	if (__lst__ != NULL) { \
+		vtp0_t __strv__ = {0}; \
+		long __i__; \
 		for(__n__ = rnd_conflist_first(__lst__); __n__ != NULL; __n__ = rnd_conflist_next(__n__)) { \
-			char *__path__; \
 			const char **__in__ = __n__->val.string; \
 			if (__in__ == NULL) \
 				continue; \
-			rnd_path_resolve(hidlib, *__in__, &__path__, 0, rnd_false); \
+			vtp0_append(&__strv__, rnd_strdup(*__in__)); \
+		} \
+		for(__i__ = 0; __i__ < __strv__.used; __i__++) { \
+			char *__path__; \
+			char *__inpa__ = __strv__.array[__i__]; \
+			rnd_path_resolve(design, __inpa__, &__path__, 0, rnd_false); \
 			res = call; \
 			free(__path__); \
 			if (res == 0) \
 				break; \
 		} \
+		for(__i__ = 0; __i__ < __strv__.used; __i__++) \
+			free(__strv__.array[__i__]); \
+		vtp0_uninit(&__strv__); \
 	} \
 } while(0)
 
@@ -364,8 +390,8 @@ do { \
 lht_node_t *rnd_conf_lht_get_first(rnd_conf_role_t target, int create);
 lht_node_t *rnd_conf_lht_get_first_crpol(rnd_conf_role_t target, rnd_conf_policy_t pol, int create);
 
-/* [3.2.0] Get first config subtree for role, create the file if doesn't exist */
-lht_node_t *rnd_pref_ensure_conf_root(rnd_hidlib_t *hidlib, rnd_conf_role_t role);
+/* Get first config subtree for role, create the file if doesn't exist */
+lht_node_t *rnd_pref_ensure_conf_root(rnd_design_t *design, rnd_conf_role_t role);
 
 /* loop helper */
 rnd_conf_listitem_t *rnd_conf_list_first_str(rnd_conflist_t *list, const char **item_str, int *idx);
@@ -396,7 +422,7 @@ lht_node_t *rnd_conf_lht_get_at(rnd_conf_role_t target, const char *path, int cr
 lht_node_t *rnd_conf_lht_get_at_mainplug(rnd_conf_role_t target, const char *path, int allow_plug, int create);
 
 /* Write an existing conf subtree to a file */
-int rnd_conf_export_to_file(rnd_hidlib_t *hidlib, const char *fn, rnd_conf_role_t role, const char *conf_path);
+int rnd_conf_export_to_file(rnd_design_t *design, const char *fn, rnd_conf_role_t role, const char *conf_path);
 
 /* Determine the policy and priority of a config lihata node;
    returns 0 on success but may not fill in both values, caller is
@@ -420,7 +446,9 @@ const char *rnd_conf_get_project_conf_name(const char *project_fn, const char *d
    subtree if needed */
 lht_node_t *rnd_conf_lht_get_first_pol(rnd_conf_role_t target, rnd_conf_policy_t pol, int create);
 
-/* (un)register a custom config file internal (executable-embedded) version;
+/* DO NOT CALL THESE DIRECLY from plugoind, use rnd_conf_plug_(un)reg()
+   instead, which are multi-design-safe.
+   (un)register a custom config file internal (executable-embedded) version;
    it's not strdup'd, the caller needs to keep the string available
    until rnd_conf_unreg_file(). */
 void rnd_conf_reg_intern(const char *intern);
@@ -438,11 +466,12 @@ void rnd_conf_files_uninit(void);
 int rnd_conf_merge_all(const char *path);
 
 /* used for ignoring specific conf nodes from specific conf sources for
-   security reason and to wanr only once */
+   security reason and to warn only once */
 struct rnd_conf_ignore_s {
 	const char *name;
 	int len;
 	int warned;
+	const char *msg; /* optional error message; if not NULL, print this */
 };
 
 
@@ -460,4 +489,11 @@ int rnd_conf_resolve(rnd_conf_resolve_t *res);
 int rnd_conf_resolve_all(rnd_conf_resolve_t *res);
 
 
+/*** misc ***/
+void rnd_conf_postproc(void);
+
+/*** Internal use ***/
+extern long rnd_conf_lht_edits[RND_CFR_max_alloc];
+extern int rnd_conf_edits_lock;
+
 #endif
