@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -28,6 +28,7 @@
 #define RND_HIDLIB_H
 
 #include <librnd/core/global_typedefs.h>
+#include <genlist/gendlist.h>
 
 typedef struct rnd_mark_s {
 	rnd_bool status;
@@ -42,12 +43,15 @@ typedef struct rnd_mark_s {
 	rnd_coord_t spare_c1, spare_c2, spare_c3, spare_c4;
 } rnd_mark_t;
 
-struct rnd_hidlib_s {
+struct rnd_design_s {
 	rnd_coord_t grid;                  /* grid resolution */
 	rnd_coord_t grid_ox, grid_oy;      /* grid offset */
-	rnd_coord_t size_x, size_y;        /* drawing area extents (or board dimensions) */
+	rnd_box_t dwg;                     /* drawing area extents (or design dimensions) */
 	char *name;                        /* name of the design */
-	char *filename;                    /* name of the file (from load) */
+	char *loadname;                    /* name of the file (from load; full path after save-as, written in by the app) */
+	char *fullpath;                    /* loadname resolved with realpath() - the actual full path file name on the file system */
+
+	rnd_project_t *project;            /* if the design is appended to any project */
 
 	/* tool state */
 	rnd_coord_t ch_x, ch_y, tool_x, tool_y; /* ch is crosshair */
@@ -56,6 +60,10 @@ struct rnd_hidlib_s {
 	rnd_mark_t tool_grabbed;                /* point where a drag&drop operation started */
 	rnd_box_t *tool_snapped_obj_bbox;
 
+	/* multi-sheet (app->multi_design==1) support */
+	gdl_elem_t link;                        /* linked list of designs currently open */
+	rnd_conf_state_t *saved_rnd_conf;       /* global config structs are saved here */
+
 	/* internal */
 	int *batch_ask_ovr;                /* if not NULL, override local ask-overwrite state - useful when manu operations that need to write files are ran in batch, e.g. in a cam job */
 
@@ -67,6 +75,9 @@ struct rnd_hidlib_s {
 	rnd_coord_t spare_c1, spare_c2, spare_c3, spare_c4;
 };
 
+#define rnd_dwg_get_size_x(dsg) ((dsg)->dwg.X2 - (dsg)->dwg.X1)
+#define rnd_dwg_get_size_y(dsg) ((dsg)->dwg.Y2 - (dsg)->dwg.Y1)
+
 /* Global application callbacks and configuration: these do not depend on
    the designbeing edited, but on the application implementation. The
    app is required to fill in global rnd_app fields before calling
@@ -79,6 +90,8 @@ typedef struct rnd_app_s {
 	const char *revision; /* VCS revision info, e.g. svn rev number */
 	const char *url;
 
+	/* when non-zero, allow multiple design files to be open at once */
+	unsigned multi_design:1;
 
 	/* menu file */
 	const char **menu_file_paths;      /* optional: NULL terminated list of search paths for the menu file */
@@ -103,33 +116,49 @@ typedef struct rnd_app_s {
 	/* Optional: if not NULL, called after conf updates to give conf_core a chance to update dynamic entries */
 	void (*conf_core_postproc)(void);
 
+	/* Optional: if not NULL, and returns non-zero, the given
+	   conf value from the given source is ignored (not merged). Role
+	   is really rnd_conf_role_t and policy is rnd_conf_policy_t, lhtn is
+	   (lht_node_t *). Typical example: project file's settings for the list
+	   of designs for the project (e.g. list of sheets in sch-rnd) should be
+	   accepted only from the project file (only when role is CFR_PROJECT). */
+	int (*conf_dont_merge_node)(const char *path, void *lhtn, int role, int default_prio, int default_policy, rnd_conf_native_t *target);
+
+	/* Optional: a NULL terminated array of config paths that shall not be
+	   loaded from desing or project files. This is for security reasons:
+	   any node that may execute code or scripts should be listed here */
+	rnd_conf_ignore_t *conf_prj_dsg_ignores;
+
+	/* Optional: a NULL terminated array of config paths that should not
+	   generate an error message when found in a file; should list old nodes
+	   that got removed from the conf system but old files could still have them */
+	rnd_conf_ignore_t *conf_ignores;
 
 	/*** callbacks ***/
 	/* Optional: called to update crosshair-attached object because crosshair coords likely changed; if NULL, rnd_tool_adjust_attached() is called instead (most apps want that) */
-	void (*adjust_attached_objects)(rnd_hidlib_t *hl); 
+	void (*adjust_attached_objects)(rnd_design_t *hl);
 
 /* Optional: Suspend the crosshair: save all crosshair states in a newly
    allocated and returned temp buffer, then reset the crosshair to initial
    state; the returned buffer is used to restore the crosshair states later
    on. Used in the get location loop. */
-	void *(*crosshair_suspend)(rnd_hidlib_t *hl);
-	void (*crosshair_restore)(rnd_hidlib_t *hl, void *susp_data);
+	void *(*crosshair_suspend)(rnd_design_t *hl);
+	void (*crosshair_restore)(rnd_design_t *hl, void *susp_data);
 
-/* Optional: move the crosshair to an absolute x;y coord on the board and
+/* Optional: move the crosshair to an absolute x;y coord on the design and
    update the GUI; if mouse_mot is non-zero, the request is a direct result
    of a mouse motion event */
-	void (*crosshair_move_to)(rnd_hidlib_t *hl, rnd_coord_t abs_x, rnd_coord_t abs_y, int mouse_mot);
+	void (*crosshair_move_to)(rnd_design_t *hl, rnd_coord_t abs_x, rnd_coord_t abs_y, int mouse_mot);
 
 /* Optional: draw any fixed mark on XOR overlay; if inhibit_drawing_mode is
    true, do not call ->set_drawing_mode */
-	void (*draw_marks)(rnd_hidlib_t *hidlib, rnd_bool inhibit_drawing_mode);
-
+	void (*draw_marks)(rnd_design_t *design, rnd_bool inhibit_drawing_mode);
 
 	/* Draw any mark following the crosshair on XOR overlay; if inhibit_drawing_mode is true, do not call ->set_drawing_mode */
-	void (*draw_attached)(rnd_hidlib_t *hidlib, rnd_bool inhibit_drawing_mode);
+	void (*draw_attached)(rnd_design_t *design, rnd_bool inhibit_drawing_mode);
 
 	/*** One of these two functions will be called whenever (parts of) the screen
-     needs redrawing (on screen, print or export, board or preview). The expose
+     needs redrawing (on screen, print or export, design or preview). The expose
      function does the following:
       - allocate any GCs needed
       - set drawing mode
@@ -142,47 +171,50 @@ typedef struct rnd_app_s {
 	void (*expose_main)(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *region, rnd_xform_t *xform_caller);
 
 	/* Mandatory: preview expose: generic, dialog based, used in preview
-	   widgets */
-	void (*expose_preview)(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *e);
-
-	/* [3.2.0] Optional: if not NULL, and returns non-zero, the given
-	   conf value from the given source is ignored (not merged). Role
-	   is really rnd_conf_role_t and policy is rnd_conf_policy_t, lhtn is
-	   (lht_node_t *). */
-	int (*conf_dont_merge_node)(const char *path, void *lhtn, int role, int default_prio, int default_policy, rnd_conf_native_t *target);
+	   widgets; e is not const because the call chain needs to fill in e->design */
+	void (*expose_preview)(rnd_hid_t *hid, rnd_hid_expose_ctx_t *e);
 
 	/* Spare: see doc/developer/spare.txt */
-	void (*spare_f2)(void), (*spare_f3)(void), (*spare_f4)(void), (*spare_f5)(void), (*spare_f6)(void);
-
-	/* [3.2.0] when non-zero, allow multiple design files to be open at once */
-	long multi_design;
-
-	long spare_l2, spare_l3, spare_l4;
-
-	/* Optional: a NULL terminated array of config paths that shall not be
-	   loaded from desing or project files. This is for security reasons:
-	   any node that may execute code or scripts should be listed here */
-	rnd_conf_ignore_t *conf_board_ignores;
-
-	/* Optional: a NULL terminated array of config paths that should not
-	   generate an error message when found in a file; should list old nodes
-	   that got removed from the conf system but old files could still have them */
-	rnd_conf_ignore_t *conf_ignores;
-
-	void *spare_p3, *spare_p4;
+	void (*spare_f1)(void), (*spare_f2)(void), (*spare_f3)(void), (*spare_f4)(void), (*spare_f5)(void), (*spare_f6)(void);
+	long spare_l1, spare_l2, spare_l3, spare_l4;
+	void *spare_p1, *spare_p2, *spare_p3, *spare_p4;
 	double spare_d1, spare_d2, spare_d3, spare_d4;
 } rnd_app_t;
 
 extern rnd_app_t rnd_app;
 
-void rnd_hidlib_event_uninit(void);
-void rnd_hidlib_event_init(void);
+/* set to 1 when librnd is running the main loop (set by hid_init.c) */
+extern int rnd_hid_in_main_loop;
 
 /* print pending log messages to stderr after gui uninit */
 void rnd_log_print_uninit_errs(const char *title);
 
-/*** API wrappers provided for plugins (these are translated into calls to the
-     app within the hidlib and are typically called from within librnd) ***/
-void rnd_hidcore_crosshair_move_to(rnd_hidlib_t *hidlib, rnd_coord_t abs_x, rnd_coord_t abs_y, int mouse_mot);
+
+/*** multi ***/
+
+/* Single-design app has loaded a new design and switched to it; no configs are
+   saved/loaded because the old design is discarded anyway, but the GUI is notified */
+void rnd_single_switch_to(rnd_design_t *dsg);
+
+/* Return the next (or previous) sheet or NULL if sheet was the last
+   sheet open. If sheet is NULL, use currently active sheet. */
+rnd_design_t *rnd_multi_neighbour_sheet(rnd_design_t *dsg);
+
+/* Return currently active design or NULL */
+rnd_design_t *rnd_multi_get_current(void);
+
+/* Swap current design to dsg; inform the GUI. If dsg is NULL, switch to
+   no-design (use only temporarily). */
+rnd_design_t *rnd_multi_switch_to(rnd_design_t *dsg);
+
+/* Switch to next or previous design */
+void rnd_multi_switch_to_delta(rnd_design_t *curr, int step);
+
+/* Return next or previous design - useful for picking one to activate
+   when dsg is unloaded */
+rnd_design_t *rnd_multi_neighbour_sheet(rnd_design_t *dsg);
+
+/* Swap current design with less side effects */
+void rnd_multi_switch_to_(rnd_design_t *dsg);
 
 #endif
