@@ -4,7 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2011 Andrew Poelstra
- *  Copyright (C) 2016,2019 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016,2019,2022 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,15 +21,10 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
- *
- *  Old contact info:
- *  Andrew Poelstra, 16966 60A Ave, V3S 8X5 Surrey, BC, Canada
- *  asp11@sfu.ca
- *
  */
 
 /* Implementation of printf wrapper to output pcb coords and angles
@@ -84,7 +79,7 @@ static int do_trunc0(char *str)
 
 
 /* Create sprintf specifier, using default_prec no precision is given */
-static inline void make_printf_spec(char *printf_spec_new, const char *printf_spec, int precision, int *trunc0)
+RND_INLINE void make_printf_spec(char *printf_spec_new, const char *printf_spec, int precision, int *trunc0)
 {
 	int i = 0;
 
@@ -111,7 +106,7 @@ do { \
 } while(0)
 
 /* append a suffix, with or without space */
-static int inline append_suffix(gds_t *dest, enum rnd_suffix_e suffix_type, const char *suffix)
+RND_INLINE int append_suffix(gds_t *dest, rnd_unit_suffix_t suffix_type, const char *suffix)
 {
 	switch (suffix_type) {
 	case RND_UNIT_NO_SUFFIX:
@@ -142,8 +137,7 @@ static int inline append_suffix(gds_t *dest, enum rnd_suffix_e suffix_type, cons
  *
  * return 0 on success, -1 on error
  */
-static int CoordsToString(gds_t *dest, rnd_coord_t coord[], int n_coords, const gds_t *printf_spec_, enum rnd_allow_e allow,
-														 enum rnd_suffix_e suffix_type)
+static int CoordsToString(gds_t *dest, rnd_coord_t coord[], int n_coords, const gds_t *printf_spec_, rnd_unit_allow_t allow, rnd_unit_suffix_t suffix_type)
 {
 	char filemode_buff[128]; /* G_ASCII_DTOSTR_BUF_SIZE */
 	char printf_spec_new_local[256];
@@ -172,9 +166,9 @@ static int CoordsToString(gds_t *dest, rnd_coord_t coord[], int n_coords, const
 		printf_spec_new = printf_spec_new_local;
 
 	/* Check our freedom in choosing units */
-	if ((allow & RND_UNIT_ALLOW_IMPERIAL) == 0)
+	if (((allow & RND_UNIT_ALLOW_IMPERIAL) == 0) && (allow < RND_UNIT_ALLOW_dyn))
 		family = RND_UNIT_METRIC;
-	else if ((allow & RND_UNIT_ALLOW_METRIC) == 0)
+	else if (((allow & RND_UNIT_ALLOW_METRIC) == 0) && (allow < RND_UNIT_ALLOW_dyn))
 		family = RND_UNIT_IMPERIAL;
 	else {
 		int met_votes = 0, imp_votes = 0;
@@ -207,30 +201,44 @@ static int CoordsToString(gds_t *dest, rnd_coord_t coord[], int n_coords, const
 
 	/* Determine scale factor -- find smallest unit that brings
 	 * the whole group above unity */
-	for (n = 0; n < rnd_get_n_units(0); ++n) {
-		if ((rnd_units[n].allow & allow) != 0 && (rnd_units[n].family == family)) {
+	for (n = 0; n < rnd_get_n_units(); ++n) {
+		const rnd_unit_t *u = rnd_units.array[n];
+		if (u->is_alias)
+			continue;
+		if (((u->allow & allow) != 0) && (u->family == family)) {
 			int n_above_one = 0;
 
 			for (i = 0; i < n_coords; ++i)
-				if (fabs(value[i] * rnd_units[n].scale_factor) > 1)
+				if (fabs(value[i] * u->scale_factor) > 1)
 					++n_above_one;
 			if (n_above_one == n_coords)
 				break;
 		}
 	}
 	/* If nothing worked, wind back to the smallest allowable unit */
-	if (n == rnd_get_n_units(0)) {
+	if (n == rnd_get_n_units()) {
+		const rnd_unit_t *u;
 		do {
 			--n;
-		} while ((n>=0) && ((rnd_units[n].allow & allow) == 0 || rnd_units[n].family != family));
+			if (n < 0)
+				break;
+			u = rnd_units.array[n];
+		} while (u->is_alias || ((u->allow & allow) == 0) || (u->family != family));
 	}
 
+	if (n < 0)
+		return -1;
+
 	/* Apply scale factor */
-	suffix = rnd_units[n].suffix;
-	for (i = 0; i < n_coords; ++i)
-		value[i] = value[i] * rnd_units[n].scale_factor;
+	{
+		const rnd_unit_t *u = rnd_units.array[n];
+		suffix = u->suffix;
+		for(i = 0; i < n_coords; ++i)
+			value[i] = value[i] * u->scale_factor;
+
+		make_printf_spec(printf_spec_new, printf_spec, u->default_prec, &trunc0);
+	}
 
-	make_printf_spec(printf_spec_new, printf_spec, rnd_units[n].default_prec, &trunc0);
 
 	/* Actually sprintf the values in place
 	 *  (+ 2 skips the ", " for first value) */
@@ -269,11 +277,11 @@ err:;
 typedef struct {
 	int           score_factor;
 
-	enum rnd_allow_e  base;
+	rnd_unit_allow_t  base;
 	double        down_limit;
-	enum rnd_allow_e  down;
+	rnd_unit_allow_t  down;
 	double        up_limit;
-	enum rnd_allow_e  up;
+	rnd_unit_allow_t  up;
 
 	/* persistent, calculated once */
 	const rnd_unit_t    *base_unit, *down_unit, *up_unit;
@@ -286,7 +294,7 @@ static human_coord_t human_coord[] = {
 };
 #define NUM_HUMAN_COORD (sizeof(human_coord) / sizeof(human_coord[0]))
 
-static inline int try_human_coord(rnd_coord_t coord, const rnd_unit_t *unit, double down_limit, double up_limit, int score_factor, double *value, unsigned int *best, const char **suffix)
+RND_INLINE int try_human_coord(rnd_coord_t coord, const rnd_unit_t *unit, double down_limit, double up_limit, int score_factor, double *value, unsigned int *best, const char **suffix)
 {
 	double v, frac, save;
 	long int digs, zeros;
@@ -330,7 +338,7 @@ static inline int try_human_coord(rnd_coord_t coord, const rnd_unit_t *unit, dou
 }
 
 /* Same as CoordsToString but take only one coord and print it in human readable format */
-static int CoordsToHumanString(gds_t *dest, rnd_coord_t coord, const gds_t *printf_spec_, enum rnd_suffix_e suffix_type)
+static int CoordsToHumanString(gds_t *dest, rnd_coord_t coord, const gds_t *printf_spec_, rnd_unit_suffix_t suffix_type)
 {
 	char filemode_buff[128]; /* G_ASCII_DTOSTR_BUF_SIZE */
 	char printf_spec_new_local[256];
@@ -411,7 +419,7 @@ int QstringToString(gds_t *dest, const char *qstr, char q, char esc, const char
 	return 0;
 }
 
-static void print_fgw_arg(gds_t *string, fgw_arg_t a, enum rnd_allow_e mask)
+static void print_fgw_arg(gds_t *string, fgw_arg_t a, rnd_unit_allow_t mask)
 {
 	if ((a.type & FGW_STR) == FGW_STR) {
 		gds_append_str(string, a.val.str);
@@ -461,6 +469,8 @@ static void print_fgw_arg(gds_t *string, fgw_arg_t a, enum rnd_allow_e mask)
 	rnd_append_printf(string, "<invalid fgw_arg_t %d>", a.type);
 }
 
+rnd_aft_ret_t (*rnd_printf_app_format)(gds_t *string, gds_t *spec, const char **fmt, rnd_unit_allow_t mask, rnd_unit_suffix_t suffix, rnd_aft_arg_t *arg, rnd_aft_arg_t *cookie) = NULL;
+
 /* Main low level pcb-printf function
  * This is a printf wrapper that accepts new format specifiers to
  * output pcb coords as various units. See the comment at the top
@@ -473,24 +483,29 @@ static void print_fgw_arg(gds_t *string, fgw_arg_t a, enum rnd_allow_e mask)
  * return 0 on success */
 int rnd_safe_append_vprintf(gds_t *string, rnd_safe_printf_t safe, const char *fmt, va_list args)
 {
+	char spec_buff[128]; /* large enough for any reasonable format spec */
 	gds_t spec;
 	const char *qstr, *needsq;
 	char tmp[128]; /* large enough for rendering a long long int */
 	int tmplen, retval = -1, slot_recursion = 0, mq_has_spec;
 	char *dot, *free_fmt = NULL;
-	enum rnd_allow_e mask = RND_UNIT_ALLOW_ALL_SANE;
+	rnd_unit_allow_t mask = RND_UNIT_ALLOW_ALL_SANE;
 	unsigned long maxfmts;
 
+	/* set up static allocation for spec */
+	spec.array = spec_buff;
+	spec.no_realloc = 1;
+	spec.used = 0;
+	spec.alloced = sizeof(spec_buff);
+
 	if ((safe & RND_SAFEPRINT_arg_max) > 0)
 		maxfmts = (safe & RND_SAFEPRINT_arg_max);
 	else
 		maxfmts = 1UL<<31;
 
-	gds_init(&spec);
-
 	new_fmt:;
 	while (*fmt) {
-		enum rnd_suffix_e suffix = RND_UNIT_NO_SUFFIX;
+		rnd_unit_suffix_t suffix = RND_UNIT_NO_SUFFIX;
 
 		if (*fmt == '%') {
 			const char *ext_unit = "";
@@ -566,7 +581,7 @@ int rnd_safe_append_vprintf(gds_t *string, rnd_safe_printf_t safe, const char *f
 
 			/* Get our sub-specifiers */
 			if (*fmt == '#') {
-				mask = RND_UNIT_ALLOW_CMIL;			/* This must be pcb's base unit */
+				mask = RND_UNIT_ALLOW_CMIL; /* This must be pcb's base unit */
 				fmt++;
 			}
 			if (*fmt == '$') {
@@ -579,220 +594,276 @@ int rnd_safe_append_vprintf(gds_t *string, rnd_safe_printf_t safe, const char *f
 			}
 
 			if (maxfmts == 0)
-				return -1;
+				goto err;
 			maxfmts--;
 
 			/* Tack full specifier onto specifier */
-			if (*fmt != 'm')
-				if (gds_append(&spec, *fmt) != 0) goto err;
+			if ((*fmt != 'm') && (*fmt != 'r'))
+				if (gds_append(&spec, *fmt) != 0)
+					goto err;
+
+
 			switch (*fmt) {
-				/* Printf specs */
-			case 'o':
-			case 'i':
-			case 'd':
-			case 'u':
-			case 'x':
-			case 'X':
-				if (safe & RND_SAFEPRINT_COORD_ONLY)
-					return -1;
-				if (spec.array[1] == 'l')
-					tmplen = sprintf(tmp, spec.array, va_arg(args, long));
-				else
-					tmplen = sprintf(tmp, spec.array, va_arg(args, int));
-				if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-				break;
-			case 'e':
-			case 'E':
-			case 'f':
-			case 'g':
-			case 'G':
-				if (safe & RND_SAFEPRINT_COORD_ONLY)
-					return -1;
-				if (strchr(spec.array, '*')) {
-					int prec = va_arg(args, int);
-					tmplen = sprintf(tmp, spec.array, va_arg(args, double), prec);
-				}
-				else
-					tmplen = sprintf(tmp, spec.array, va_arg(args, double));
-				if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-				break;
-			case 'c':
-				if (safe & RND_SAFEPRINT_COORD_ONLY)
-					return -1;
-				if (spec.array[1] == 'l' && sizeof(int) <= sizeof(wchar_t))
-					tmplen = sprintf(tmp, spec.array, va_arg(args, wchar_t));
-				else
-					tmplen = sprintf(tmp, spec.array, va_arg(args, int));
-				if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-				break;
-			case 's':
-				if (safe & RND_SAFEPRINT_COORD_ONLY)
-					return -1;
-				if (spec.array[0] == 'l') {
-					fprintf(stderr, "Internal error: appending %%ls is not supported\n");
-					abort();
-				}
-				else {
-					const char *s = va_arg(args, const char *);
-					if (s == NULL) s = "(null)";
-					if (gds_append_str(string, s) != 0) goto err;
-				}
-				break;
-			case 'n':
-				if (safe & RND_SAFEPRINT_COORD_ONLY)
-					return -1;
-				/* Depending on gcc settings, this will probably break with
-				 *  some silly "can't put %n in writable data space" message */
-				tmplen = sprintf(tmp, spec.array, va_arg(args, int *));
-				if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-				break;
-			case 'p':
-				if (safe & RND_SAFEPRINT_COORD_ONLY)
-					return -1;
-				tmplen = sprintf(tmp, spec.array, va_arg(args, void *));
-				if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-				break;
-			case '%':
-				if (gds_append(string, '%') != 0) goto err;
-				break;
-				/* Our specs */
-			case 'm':
-				++fmt;
-				if (*fmt == '*')
-					ext_unit = va_arg(args, const char *);
-				if (*fmt != '+' && *fmt != 'a' && *fmt != 'A' && *fmt != 'f' && *fmt != 'q' && *fmt != 'w')
-					value[0] = va_arg(args, rnd_coord_t);
-				count = 1;
-				switch (*fmt) {
-				case 'q':
-					qstr = va_arg(args, const char *);
-					if (mq_has_spec)
-						needsq = spec.array;
+
+				/*** Standard printf specs ***/
+				case 'o':
+				case 'i':
+				case 'd':
+				case 'u':
+				case 'x':
+				case 'X':
+					if (safe & RND_SAFEPRINT_COORD_ONLY)
+						goto err;
+					if (spec.array[1] == 'l')
+						tmplen = sprintf(tmp, spec.array, va_arg(args, long));
 					else
-						needsq = " \"\\";
-					if (QstringToString(string, qstr, '"', '\\', needsq) != 0) goto err;
+						tmplen = sprintf(tmp, spec.array, va_arg(args, int));
+					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
 					break;
-				case 'I':
-					if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_NM, RND_UNIT_NO_SUFFIX) != 0) goto err;
+				case 'e':
+				case 'E':
+				case 'f':
+				case 'g':
+				case 'G':
+					if (safe & RND_SAFEPRINT_COORD_ONLY)
+						goto err;
+					if (strchr(spec.array, '*')) {
+						int prec = va_arg(args, int);
+						tmplen = sprintf(tmp, spec.array, va_arg(args, double), prec);
+					}
+					else
+						tmplen = sprintf(tmp, spec.array, va_arg(args, double));
+					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
+					break;
+				case 'c':
+					if (safe & RND_SAFEPRINT_COORD_ONLY)
+						goto err;
+					if (spec.array[1] == 'l' && sizeof(int) <= sizeof(wchar_t))
+						tmplen = sprintf(tmp, spec.array, va_arg(args, wchar_t));
+					else
+						tmplen = sprintf(tmp, spec.array, va_arg(args, int));
+					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
 					break;
 				case 's':
-					if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_MM | RND_UNIT_ALLOW_MIL, suffix) != 0) goto err;
+					if (safe & RND_SAFEPRINT_COORD_ONLY)
+						goto err;
+					if (spec.array[0] == 'l') {
+						fprintf(stderr, "Internal error: appending %%ls is not supported\n");
+						abort();
+					}
+					else {
+						const char *s = va_arg(args, const char *);
+						if (s == NULL) s = "(null)";
+						if (gds_append_str(string, s) != 0) goto err;
+					}
 					break;
-				case 'S':
-					if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_NATURAL, suffix) != 0) goto err;
+				case 'n':
+					if (safe & RND_SAFEPRINT_COORD_ONLY)
+						goto err;
+					/* Depending on gcc settings, this will probably break with
+					 *  some silly "can't put %n in writable data space" message */
+					tmplen = sprintf(tmp, spec.array, va_arg(args, int *));
+					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
 					break;
-				case 'N':
-					if (CoordsToString(string, value, 1, &spec, mask, suffix) != 0) goto err;
+				case 'p':
+					if (safe & RND_SAFEPRINT_COORD_ONLY)
+						goto err;
+					tmplen = sprintf(tmp, spec.array, va_arg(args, void *));
+					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
 					break;
-				case 'H':
-					if ((safe & RND_SAFEPRINT_COORD_ONLY) && (value[0] > 1))
-						return -1;
-					if (CoordsToHumanString(string, value[0], &spec, suffix) != 0) goto err;
+				case '%':
+					if (gds_append(string, '%') != 0) goto err;
 					break;
-				case 'M':
-					if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_METRIC, suffix) != 0) goto err;
-					break;
-				case 'L':
-					if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_IMPERIAL, suffix) != 0) goto err;
-					break;
-				case 'k':
-					if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_DMIL, suffix) != 0) goto err;
+
+				/*** librnd (ex-pcb-rnd) custom spec ***/
+				case 'm':
+					++fmt;
+					if (*fmt == '*') {
+						if (safe & RND_SAFEPRINT_COORD_ONLY)
+							goto err;
+						ext_unit = va_arg(args, const char *);
+					}
+					if (*fmt != '+' && *fmt != 'a' && *fmt != 'A' && *fmt != 'f' && *fmt != 'q' && *fmt != 'w')
+						value[0] = va_arg(args, rnd_coord_t);
+					count = 1;
+					switch (*fmt) {
+						case 'q':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							qstr = va_arg(args, const char *);
+							if (mq_has_spec)
+								needsq = spec.array;
+							else
+								needsq = " \"\\";
+							if (QstringToString(string, qstr, '"', '\\', needsq) != 0) goto err;
+							break;
+						case 'I':
+							if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_NM, RND_UNIT_NO_SUFFIX) != 0) goto err;
+							break;
+						case 's':
+							if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_MM | RND_UNIT_ALLOW_MIL, suffix) != 0) goto err;
+							break;
+						case 'S':
+							if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_NATURAL, suffix) != 0) goto err;
+							break;
+						case 'N':
+							if (CoordsToString(string, value, 1, &spec, mask, suffix) != 0) goto err;
+							break;
+						case 'H':
+							if ((safe & RND_SAFEPRINT_COORD_ONLY) && (value[0] > 1))
+								goto err;
+							if (CoordsToHumanString(string, value[0], &spec, suffix) != 0) goto err;
+							break;
+						case 'M':
+							if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_METRIC, suffix) != 0) goto err;
+							break;
+						case 'L':
+							if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_IMPERIAL, suffix) != 0) goto err;
+							break;
+						case 'k':
+							if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_DMIL, suffix) != 0) goto err;
+							break;
+						case 'r':
+							if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_READABLE, RND_UNIT_NO_SUFFIX) != 0) goto err;
+							break;
+							/* All these fallthroughs are deliberate */
+						case '9':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '8':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '7':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '6':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '5':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '4':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '3':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+						case '2':
+						case 'D':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[count++] = va_arg(args, rnd_coord_t);
+							if (CoordsToString(string, value, count, &spec, mask & RND_UNIT_ALLOW_ALL_SANE, suffix) != 0) goto err;
+							break;
+						case 'd':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							value[1] = va_arg(args, rnd_coord_t);
+							if (CoordsToString(string, value, 2, &spec, RND_UNIT_ALLOW_MM | RND_UNIT_ALLOW_MIL, suffix) != 0) goto err;
+							break;
+						case '*':
+							{
+								int found = 0;
+								for (i = 0; i < rnd_get_n_units(); ++i) {
+									const rnd_unit_t *u = rnd_units.array[i];
+									if (strcmp(ext_unit, u->suffix) == 0) {
+										if (CoordsToString(string, value, 1, &spec, u->allow, suffix) != 0) goto err;
+										found = 1;
+										break;
+									}
+								}
+								if (!found)
+									if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_ALL_SANE, suffix) != 0) goto err;
+							}
+							break;
+						case 'a':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							if (gds_append_len(&spec, ".06f", 4) != 0) goto err;
+							if (suffix == RND_UNIT_SUFFIX)
+								if (gds_append_len(&spec, " deg", 4) != 0) goto err;
+							tmplen = sprintf(tmp, spec.array, (double) va_arg(args, rnd_angle_t));
+							if (gds_append_len(string, tmp, tmplen) != 0) goto err;
+							break;
+						case 'A':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							if (gds_append_len(&spec, ".0f", 3) != 0) goto err;
+							/* if (suffix == RND_UNIT_SUFFIX)
+								if (gds_append_len(&spec, " deg", 4) != 0) goto err;*/
+							tmplen = sprintf(tmp, spec.array, 10*((double) va_arg(args, rnd_angle_t))); /* kicad legacy needs decidegrees */
+							if (gds_append_len(string, tmp, tmplen) != 0) goto err;
+							break;
+						case 'f':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							gds_append_len(&spec, "f", 1);
+							tmplen = sprintf(tmp, spec.array, va_arg(args, double));
+							dot = strchr(spec.array, '.');
+							if ((dot != NULL) && (dot[1] == '0'))
+								tmplen = do_trunc0(tmp);
+							if (gds_append_len(string, tmp, tmplen) != 0) goto err;
+							break;
+						case 'w':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							print_fgw_arg(string, va_arg(args, fgw_arg_t), mask);
+							break;
+						case '+':
+							if (safe & RND_SAFEPRINT_COORD_ONLY)
+								goto err;
+							mask = va_arg(args, rnd_unit_allow_t);
+							break;
+						default:
+							{
+								int found = 0;
+								for (i = 0; i < rnd_get_n_units(); ++i) {
+									const rnd_unit_t *u = rnd_units.array[i];
+									if (*fmt == u->printf_code) {
+										if (CoordsToString(string, value, 1, &spec, u->allow, suffix) != 0) goto err;
+										found = 1;
+										break;
+									}
+								}
+								if (!found)
+									if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_ALL_SANE, suffix) != 0) goto err;
+							}
+							break;
+					}
 					break;
+
+				/*** ringdove app-specific custom spec ***/
 				case 'r':
-					if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_READABLE, RND_UNIT_NO_SUFFIX) != 0) goto err;
-					break;
-					/* All these fallthroughs are deliberate */
-				case '9':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '8':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '7':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '6':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '5':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '4':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '3':
-					value[count++] = va_arg(args, rnd_coord_t);
-				case '2':
-				case 'D':
-					if (safe & RND_SAFEPRINT_COORD_ONLY)
-						return -1;
-					value[count++] = va_arg(args, rnd_coord_t);
-					if (CoordsToString(string, value, count, &spec, mask & RND_UNIT_ALLOW_ALL_SANE, suffix) != 0) goto err;
-					break;
-				case 'd':
-					if (safe & RND_SAFEPRINT_COORD_ONLY)
-						return -1;
-					value[1] = va_arg(args, rnd_coord_t);
-					if (CoordsToString(string, value, 2, &spec, RND_UNIT_ALLOW_MM | RND_UNIT_ALLOW_MIL, suffix) != 0) goto err;
-					break;
-				case '*':
 					{
-						int found = 0;
-						for (i = 0; i < rnd_get_n_units(1); ++i) {
-							if (strcmp(ext_unit, rnd_units[i].suffix) == 0) {
-								if (CoordsToString(string, value, 1, &spec, rnd_units[i].allow, suffix) != 0) goto err;
-								found = 1;
-								break;
-							}
+						rnd_aft_ret_t r;
+						rnd_aft_arg_t arg, cookie;
+
+						if (rnd_printf_app_format == NULL)
+							goto err;
+						++fmt;
+						memset(&cookie, 0, sizeof(cookie));
+						r = rnd_printf_app_format(string, &spec, &fmt, mask, suffix, NULL, &cookie);
+						switch(r) {
+							case RND_AFT_INT:       arg.i = va_arg(args, int); break;
+							case RND_AFT_LONG:      arg.l = va_arg(args, long); break;
+							case RND_AFT_DOUBLE:    arg.d = va_arg(args, double); break;
+							case RND_AFT_PTR:       arg.p = va_arg(args, void *); break;
+							case RND_AFT_error:
+							default:
+								goto err;
 						}
-						if (!found)
-							if (CoordsToString(string, value, 1, &spec, mask & RND_UNIT_ALLOW_ALL_SANE, suffix) != 0) goto err;
+						r = rnd_printf_app_format(string, &spec, &fmt, mask, suffix, &arg, &cookie);
+						if (r == RND_AFT_error)
+							goto err;
 					}
 					break;
-				case 'a':
-					if (safe & RND_SAFEPRINT_COORD_ONLY)
-						return -1;
-					if (gds_append_len(&spec, ".06f", 4) != 0) goto err;
-					if (suffix == RND_UNIT_SUFFIX)
-						if (gds_append_len(&spec, " deg", 4) != 0) goto err;
-					tmplen = sprintf(tmp, spec.array, (double) va_arg(args, rnd_angle_t));
-					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-					break;
-				case 'A':
-					if (safe & RND_SAFEPRINT_COORD_ONLY)
-						return -1;
-					if (gds_append_len(&spec, ".0f", 3) != 0) goto err;
-					/* if (suffix == RND_UNIT_SUFFIX)
-						if (gds_append_len(&spec, " deg", 4) != 0) goto err;*/
-					tmplen = sprintf(tmp, spec.array, 10*((double) va_arg(args, rnd_angle_t))); /* kicad legacy needs decidegrees */
-					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-					break;
-				case 'f':
-					if (safe & RND_SAFEPRINT_COORD_ONLY)
-						return -1;
-					gds_append_len(&spec, "f", 1);
-					tmplen = sprintf(tmp, spec.array, va_arg(args, double));
-					dot = strchr(spec.array, '.');
-					if ((dot != NULL) && (dot[1] == '0'))
-						tmplen = do_trunc0(tmp);
-					if (gds_append_len(string, tmp, tmplen) != 0) goto err;
-					break;
-				case 'w':
-					print_fgw_arg(string, va_arg(args, fgw_arg_t), mask);
-					break;
-				case '+':
-					mask = va_arg(args, enum rnd_allow_e);
-					break;
-				default:
-					{
-						int found = 0;
-						for (i = 0; i < rnd_get_n_units(1); ++i) {
-							if (*fmt == rnd_units[i].printf_code) {
-								if (CoordsToString(string, value, 1, &spec, rnd_units[i].allow, suffix) != 0) goto err;
-								found = 1;
-								break;
-							}
-						}
-						if (!found)
-							if (CoordsToString(string, value, 1, &spec, RND_UNIT_ALLOW_ALL_SANE, suffix) != 0) goto err;
-					}
-					break;
-				}
-				break;
 			}
 		}
 		else
@@ -802,7 +873,7 @@ int rnd_safe_append_vprintf(gds_t *string, rnd_safe_printf_t safe, const char *f
 
 	retval = 0;
 err:;
-	gds_uninit(&spec);
+	/* gds_uninit(&spec); - not needed with static allocation */
 	if (free_fmt != NULL)
 		free(free_fmt);
 
