@@ -1,13 +1,13 @@
 #include "config.h"
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 
 #include <stdio.h>
 
-#include <librnd/core/grid.h>
+#include <librnd/hid/grid.h>
 #include <librnd/core/color.h>
 #include <librnd/core/color_cache.h>
-#include <librnd/core/hid_attrib.h>
-#include <librnd/core/pixmap.h>
+#include <librnd/hid/hid_attrib.h>
+#include <librnd/hid/pixmap.h>
 #include <librnd/core/globalconst.h>
 
 #include <librnd/plugins/lib_gtk2_common/compat.h>
@@ -101,7 +101,7 @@ static rnd_bool map_color(const rnd_color_t *inclr, rnd_gtk_color_t *color)
 }
 
 
-static int ghid_gdk_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const char *purpose, int purpi, rnd_layer_id_t layer, unsigned int flags, int is_empty, rnd_xform_t **xform)
+static int ghid_gdk_set_layer_group(rnd_hid_t *hid, rnd_design_t *design, rnd_layergrp_id_t group, const char *purpose, int purpi, rnd_layer_id_t layer, unsigned int flags, int is_empty, rnd_xform_t **xform)
 {
 	/* draw anything */
 	return 1;
@@ -136,7 +136,7 @@ static void set_clip(render_priv_t *priv, GdkGC *gc)
 		gdk_gc_set_clip_mask(gc, NULL);
 }
 
-static inline void ghid_gdk_draw_grid_global(rnd_hidlib_t *hidlib)
+static inline void ghid_gdk_draw_grid_global(rnd_design_t *hidlib)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
 	rnd_coord_t x, y, x1, y1, x2, y2, grd;
@@ -144,19 +144,19 @@ static inline void ghid_gdk_draw_grid_global(rnd_hidlib_t *hidlib)
 	static GdkPoint *points = NULL, *points3 = NULL;
 	static int npoints = 0, npoints3 = 0;
 
-	x1 = rnd_grid_fit(RND_CLAMP(SIDE_X(&ghidgui->port.view, ghidgui->port.view.x0), 0, hidlib->size_x), hidlib->grid, hidlib->grid_ox);
-	y1 = rnd_grid_fit(RND_CLAMP(SIDE_Y(&ghidgui->port.view, ghidgui->port.view.y0), 0, hidlib->size_y), hidlib->grid, hidlib->grid_oy);
-	x2 = rnd_grid_fit(RND_CLAMP(SIDE_X(&ghidgui->port.view, ghidgui->port.view.x0 + ghidgui->port.view.width - 1), 0, hidlib->size_x), hidlib->grid, hidlib->grid_ox);
-	y2 = rnd_grid_fit(RND_CLAMP(SIDE_Y(&ghidgui->port.view, ghidgui->port.view.y0 + ghidgui->port.view.height - 1), 0, hidlib->size_y), hidlib->grid, hidlib->grid_oy);
+	x1 = rnd_grid_fit(RND_CLAMP(SIDE_X(&ghidgui->port.view, ghidgui->port.view.x0), hidlib->dwg.X1, hidlib->dwg.X2), hidlib->grid, hidlib->grid_ox);
+	y1 = rnd_grid_fit(RND_CLAMP(SIDE_Y(&ghidgui->port.view, ghidgui->port.view.y0), hidlib->dwg.Y1, hidlib->dwg.Y2), hidlib->grid, hidlib->grid_oy);
+	x2 = rnd_grid_fit(RND_CLAMP(SIDE_X(&ghidgui->port.view, ghidgui->port.view.x0 + ghidgui->port.view.width - 1), hidlib->dwg.X1, hidlib->dwg.X2), hidlib->grid, hidlib->grid_ox);
+	y2 = rnd_grid_fit(RND_CLAMP(SIDE_Y(&ghidgui->port.view, ghidgui->port.view.y0 + ghidgui->port.view.height - 1), hidlib->dwg.Y1, hidlib->dwg.Y2), hidlib->grid, hidlib->grid_oy);
 
 	grd = hidlib->grid;
 	if (grd <= 0)
 		grd = 1;
 
-	if (Vz(grd) < rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.min_dist_px) {
-		if (!rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.sparse)
+	if (Vz(grd) < rnd_conf.editor.global_grid.min_dist_px) {
+		if (!rnd_conf.editor.global_grid.sparse)
 			return;
-		grd *= (rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.min_dist_px / Vz(grd));
+		grd *= (rnd_conf.editor.global_grid.min_dist_px / Vz(grd));
 	}
 
 	if (x1 > x2) {
@@ -233,7 +233,7 @@ static inline void ghid_gdk_draw_grid_global(rnd_hidlib_t *hidlib)
 	}
 }
 
-static void ghid_gdk_draw_grid_local_(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t cy, int radius)
+static void ghid_gdk_draw_grid_local_(rnd_design_t *hidlib, rnd_coord_t cx, rnd_coord_t cy, int radius)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
 	static GdkPoint *points_base = NULL;
@@ -316,14 +316,14 @@ static void ghid_gdk_draw_grid_local_(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_
 static int grid_local_have_old = 0, grid_local_old_r = 0;
 static rnd_coord_t grid_local_old_x, grid_local_old_y;
 
-static void ghid_gdk_draw_grid_local(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t cy)
+static void ghid_gdk_draw_grid_local(rnd_design_t *hidlib, rnd_coord_t cx, rnd_coord_t cy)
 {
 	if (grid_local_have_old) {
 		ghid_gdk_draw_grid_local_(hidlib, grid_local_old_x, grid_local_old_y, grid_local_old_r);
 		grid_local_have_old = 0;
 	}
 
-	if (!rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.enable)
+	if (!rnd_conf.editor.local_grid.enable)
 		return;
 
 	if ((Vz(hidlib->grid) < RND_MIN_GRID_DISTANCE) || (!rnd_conf.editor.draw_grid))
@@ -334,13 +334,13 @@ static void ghid_gdk_draw_grid_local(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_c
 	cy = (cy / hidlib->grid) * hidlib->grid + hidlib->grid_oy;
 
 	grid_local_have_old = 1;
-	ghid_gdk_draw_grid_local_(hidlib, cx, cy, rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.radius);
+	ghid_gdk_draw_grid_local_(hidlib, cx, cy, rnd_conf.editor.local_grid.radius);
 	grid_local_old_x = cx;
 	grid_local_old_y = cy;
-	grid_local_old_r = rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.radius;
+	grid_local_old_r = rnd_conf.editor.local_grid.radius;
 }
 
-static void ghid_gdk_draw_grid(rnd_hidlib_t *hidlib)
+static void ghid_gdk_draw_grid(rnd_design_t *hidlib)
 {
 	static GdkColormap *colormap = NULL;
 	render_priv_t *priv = ghidgui->port.render_priv;
@@ -366,7 +366,7 @@ static void ghid_gdk_draw_grid(rnd_hidlib_t *hidlib)
 		set_clip(priv, priv->grid_gc);
 	}
 
-	if (rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.enable) {
+	if (rnd_conf.editor.local_grid.enable) {
 		ghid_gdk_draw_grid_local(hidlib, grid_local_old_x, grid_local_old_y);
 		return;
 	}
@@ -379,7 +379,7 @@ static void ghid_gdk_draw_grid(rnd_hidlib_t *hidlib)
 /* write 1 to dst for any pixel in SRC that is non-transparent */
 void copy_mask_pixmap(GdkPixmap *DST, GdkPixbuf *SRC, int sx, int sy, GdkGC *gc)
 {
-	int src_rowstd, dst_rowstd, nch, x, y, subp;
+	int src_rowstd, nch, x, y;
 	unsigned char *src_row, *src;
 
 	src_row = gdk_pixbuf_get_pixels(SRC);
@@ -397,7 +397,7 @@ void copy_mask_pixmap(GdkPixmap *DST, GdkPixbuf *SRC, int sx, int sy, GdkGC *gc)
 	}
 }
 
-static void ghid_gdk_uninit_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm)
+static void ghid_gdk_uninit_pixmap(rnd_design_t *hidlib, rnd_gtk_pixmap_t *gpm)
 {
 	if (gpm->cache.gdk.pb != NULL) {
 		g_object_unref(G_OBJECT(gpm->cache.gdk.pb));
@@ -409,7 +409,7 @@ static void ghid_gdk_uninit_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm)
 	}
 }
 
-static void ghid_gdk_draw_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t dw, rnd_coord_t dh)
+static void ghid_gdk_draw_pixmap(rnd_design_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t dw, rnd_coord_t dh)
 {
 	GdkInterpType interp_type;
 	gint src_x, src_y, dst_x, dst_y, w, h, w_src, h_src;
@@ -471,10 +471,10 @@ static void ghid_gdk_draw_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rn
 	}
 }
 
-static void ghid_gdk_draw_bg_image(rnd_hidlib_t *hidlib)
+static void ghid_gdk_draw_bg_image(rnd_design_t *hidlib)
 {
 	if (ghidgui->bg_pixmap.image != NULL)
-		ghid_gdk_draw_pixmap(hidlib, &ghidgui->bg_pixmap, 0, 0, hidlib->size_x, hidlib->size_y);
+		ghid_gdk_draw_pixmap(hidlib, &ghidgui->bg_pixmap, hidlib->dwg.X1, hidlib->dwg.Y2, hidlib->dwg.X2, hidlib->dwg.Y2);
 }
 
 
@@ -580,18 +580,18 @@ static void ghid_gdk_set_special_colors(rnd_conf_native_t *cfg)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
 
-	if (((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.background) && priv->bg_gc) {
-		if (map_color(&cfg->val.color[0], &priv->bg_color)) {
+	if (((cfg == NULL) || ((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.background)) && priv->bg_gc) {
+		if (map_color(&rnd_conf.appearance.color.background, &priv->bg_color)) {
 			gdk_gc_set_foreground(priv->bg_gc, &priv->bg_color);
 			set_special_grid_color();
 		}
 	}
-	else if (((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.off_limit) && priv->offlimits_gc) {
-		if (map_color(&cfg->val.color[0], &priv->offlimits_color))
+	if (((cfg == NULL) || ((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.off_limit)) && priv->offlimits_gc) {
+		if (map_color(&rnd_conf.appearance.color.off_limit, &priv->offlimits_color))
 			gdk_gc_set_foreground(priv->offlimits_gc, &priv->offlimits_color);
 	}
-	else if (((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.grid) && priv->grid_gc) {
-		if (map_color(&cfg->val.color[0], &priv->grid_color))
+	if (((cfg == NULL) || ((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.grid)) && priv->grid_gc) {
+		if (map_color(&rnd_conf.appearance.color.grid, &priv->grid_color))
 			set_special_grid_color();
 	}
 }
@@ -803,6 +803,13 @@ static void ghid_gdk_draw_arc(rnd_hid_gc_t gc, rnd_coord_t cx, rnd_coord_t cy, r
 	vrx2 = Vz(xradius*2.0);
 	vry2 = Vz(yradius*2.0);
 
+	if ((vrx2 <= 2) && (vry2 <= 2)) {
+		gdk_draw_point(priv->out_pixel, priv->pixel_gc, Vxd(cx), Vyd(cy));
+		if (priv->out_clip != NULL)
+			gdk_draw_point(priv->out_clip, priv->clip_gc, Vxd(cx), Vyd(cy));
+		return;
+	}
+
 	if ((delta_angle > 360.0) || (delta_angle < -360.0)) {
 		start_angle = 0;
 		delta_angle = 360;
@@ -1132,7 +1139,7 @@ static void ghid_gdk_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 		gdk_draw_rectangle(priv->out_clip, priv->clip_gc, TRUE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
 }
 
-static void redraw_region(rnd_hidlib_t *hidlib, GdkRectangle *rect)
+static void redraw_region(rnd_design_t *hidlib, GdkRectangle *rect)
 {
 	int eleft, eright, etop, ebottom;
 	rnd_hid_expose_ctx_t ctx;
@@ -1157,6 +1164,7 @@ static void redraw_region(rnd_hidlib_t *hidlib, GdkRectangle *rect)
 	set_clip(priv, priv->offlimits_gc);
 	set_clip(priv, priv->grid_gc);
 
+	ctx.design = hidlib;
 	ctx.view.X1 = MIN(Px(priv->clip_rect.x), Px(priv->clip_rect.x + priv->clip_rect.width + 1));
 	ctx.view.Y1 = MIN(Py(priv->clip_rect.y), Py(priv->clip_rect.y + priv->clip_rect.height + 1));
 	ctx.view.X2 = MAX(Px(priv->clip_rect.x), Px(priv->clip_rect.x + priv->clip_rect.width + 1));
@@ -1167,10 +1175,10 @@ static void redraw_region(rnd_hidlib_t *hidlib, GdkRectangle *rect)
 	ctx.view.Y1 = ctx.view.Y1;
 	ctx.view.Y2 = ctx.view.Y2;
 
-	eleft = Vx(0);
-	eright = Vx(hidlib->size_x);
-	etop = Vy(0);
-	ebottom = Vy(hidlib->size_y);
+	eleft   = Vx(hidlib->dwg.X1);
+	eright  = Vx(hidlib->dwg.X2);
+	etop    = Vy(hidlib->dwg.Y1);
+	ebottom = Vy(hidlib->dwg.Y2);
 	if (eleft > eright) {
 		int tmp = eleft;
 		eleft = eright;
@@ -1223,7 +1231,7 @@ static void redraw_region(rnd_hidlib_t *hidlib, GdkRectangle *rect)
 static int preview_lock = 0;
 static void ghid_gdk_invalidate_lr(rnd_hid_t *hid, rnd_coord_t left, rnd_coord_t right, rnd_coord_t top, rnd_coord_t bottom)
 {
-	rnd_hidlib_t *hidlib = ghidgui->hidlib;
+	rnd_design_t *hidlib = ghidgui->hidlib;
 	int dleft, dright, dtop, dbottom;
 	int minx, maxx, miny, maxy;
 	GdkRectangle rect;
@@ -1256,7 +1264,7 @@ static void ghid_gdk_invalidate_lr(rnd_hid_t *hid, rnd_coord_t left, rnd_coord_t
 
 static void ghid_gdk_invalidate_all(rnd_hid_t *hid)
 {
-	rnd_hidlib_t *hidlib = ghidgui->hidlib;
+	rnd_design_t *hidlib = ghidgui->hidlib;
 	if (ghidgui && ghidgui->topwin.menu.menu_bar) {
 		redraw_region(hidlib, NULL);
 		if (!preview_lock) {
@@ -1270,7 +1278,7 @@ static void ghid_gdk_invalidate_all(rnd_hid_t *hid)
 
 static void ghid_gdk_notify_crosshair_change(rnd_hid_t *hid, rnd_bool changes_complete)
 {
-	rnd_hidlib_t *hidlib = ghidgui->hidlib;
+	rnd_design_t *hidlib = ghidgui->hidlib;
 	render_priv_t *priv = ghidgui->port.render_priv;
 
 	/* We sometimes get called before the GUI is up */
@@ -1305,7 +1313,7 @@ static void ghid_gdk_notify_crosshair_change(rnd_hid_t *hid, rnd_bool changes_co
 
 static void ghid_gdk_notify_mark_change(rnd_hid_t *hid, rnd_bool changes_complete)
 {
-	rnd_hidlib_t *hidlib = ghidgui->hidlib;
+	rnd_design_t *hidlib = ghidgui->hidlib;
 	render_priv_t *priv = ghidgui->port.render_priv;
 
 	/* We sometimes get called before the GUI is up */
@@ -1464,10 +1472,18 @@ static gboolean ghid_gdk_drawing_area_expose_cb(GtkWidget *widget, rnd_gtk_expos
 	rnd_gtk_port_t *port = vport;
 	render_priv_t *priv = port->render_priv;
 	GdkWindow *window = gtkc_widget_get_window(ghidgui->port.drawing_area);
+	rnd_hid_t *hid_save;
+
+	/* save rnd_render and restore at the end: we may get an async expose draw
+	   while it is set to an exporter */
+	hid_save = rnd_render;
+	rnd_render = &gtk2_gdk_hid;
 
 	gdk_draw_drawable(window, priv->bg_gc, priv->base_pixel,
 		ev->area.x, ev->area.y, ev->area.x, ev->area.y, ev->area.width, ev->area.height);
 	show_crosshair(TRUE);
+
+	rnd_render = hid_save;
 	return FALSE;
 }
 
@@ -1475,7 +1491,7 @@ static void ghid_gdk_port_drawing_realize_cb(GtkWidget *widget, gpointer data)
 {
 }
 
-static gboolean ghid_gdk_preview_expose(GtkWidget *widget, rnd_gtk_expose_t *ev, rnd_hid_expose_t expcall, rnd_hid_expose_ctx_t *ctx)
+static gboolean ghid_gdk_preview_expose(GtkWidget *widget, rnd_gtk_expose_t *ev, rnd_hid_preview_expose_t expcall, rnd_hid_expose_ctx_t *ctx)
 {
 	GdkWindow *window = gtkc_widget_get_window(widget);
 	GdkDrawable *save_drawable;
@@ -1486,6 +1502,12 @@ static gboolean ghid_gdk_preview_expose(GtkWidget *widget, rnd_gtk_expose_t *ev,
 	render_priv_t *priv = ghidgui->port.render_priv;
 	rnd_coord_t ox1 = ctx->view.X1, oy1 = ctx->view.Y1, ox2 = ctx->view.X2, oy2 = ctx->view.Y2;
 	rnd_coord_t save_cpp;
+	rnd_hid_t *hid_save;
+
+	/* save rnd_render and restore at the end: we may get an async expose draw
+	   while it is set to an exporter */
+	hid_save = rnd_render;
+	rnd_render = &gtk2_gdk_hid;
 
 	vw = ctx->view.X2 - ctx->view.X1;
 	vh = ctx->view.Y2 - ctx->view.Y1;
@@ -1531,6 +1553,8 @@ static gboolean ghid_gdk_preview_expose(GtkWidget *widget, rnd_gtk_expose_t *ev,
 	ghidgui->port.view.canvas_width = save_width;
 	ghidgui->port.view.canvas_height = save_height;
 
+	rnd_render = hid_save;
+
 	return FALSE;
 }
 
