@@ -2,7 +2,7 @@
 
 #include <genlist/gendlist.h>
 #include <librnd/rnd_config.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -15,18 +15,18 @@
 #include <librnd/core/event.h>
 #include <librnd/core/rnd_printf.h>
 
-#include <librnd/core/hid.h>
+#include <librnd/hid/hid.h>
 #include "lesstif.h"
-#include <librnd/core/hid_attrib.h>
+#include <librnd/hid/hid_attrib.h>
 #include <librnd/core/actions.h>
-#include <librnd/core/hid_init.h>
+#include <librnd/hid/hid_init.h>
 #include "ltf_stdarg.h"
 #include <librnd/core/misc_util.h>
 #include "dialogs.h"
 
 
 int rnd_ltf_ok;
-extern rnd_hidlib_t *ltf_hidlib;
+extern rnd_design_t *ltf_hidlib;
 
 
 #define COMPONENT_SIDE_NAME "(top)"
@@ -671,12 +671,13 @@ static void ltf_initial_wstates(lesstif_attr_dlg_t *ctx)
 			XtUnmanageChild(ctx->wltop[n]);
 }
 
-void *lesstif_attr_dlg_new(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny)
+void lesstif_attr_dlg_new(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny, void **hid_ctx_out)
 {
 	Widget topform, main_tbl;
 	lesstif_attr_dlg_t *ctx;
 
 	ctx = calloc(sizeof(lesstif_attr_dlg_t), 1);
+	*hid_ctx_out = ctx;
 	ctx->creating = 1;
 	ctx->hidlib = ltf_hidlib;
 	ctx->attrs = attrs;
@@ -748,7 +749,6 @@ void *lesstif_attr_dlg_new(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *
 	ltf_initial_wstates(ctx);
 
 	ctx->creating = 0;
-	return ctx;
 }
 
 void *lesstif_attr_sub_new(Widget parent_box, rnd_hid_attribute_t *attrs, int n_attrs, void *caller_data)
@@ -775,6 +775,13 @@ void *lesstif_attr_sub_new(Widget parent_box, rnd_hid_attribute_t *attrs, int n_
 	return ctx;
 }
 
+void lesstif_attr_sub_update_hidlib(void *hid_ctx, rnd_design_t *new_dsg)
+{
+	lesstif_attr_dlg_t *ctx = hid_ctx;
+	ctx->hidlib = new_dsg;
+}
+
+
 int lesstif_attr_dlg_run(void *hid_ctx)
 {
 	lesstif_attr_dlg_t *ctx = hid_ctx;
@@ -952,12 +959,10 @@ void lesstif_update_layer_groups()
 TODO("layer: call a redraw on the edit group")
 }
 
-rnd_hidlib_t *ltf_attr_get_dad_hidlib(void *hid_ctx)
+rnd_design_t *ltf_attr_get_dad_hidlib(void *hid_ctx)
 {
 	lesstif_attr_dlg_t *ctx = hid_ctx;
-	if (rnd_hid_enable_per_dialog_hidlib)
-		return ctx->hidlib;
-	return ltf_hidlib;
+	return ctx->hidlib;
 }
 
 
