@@ -2,9 +2,9 @@
 
 #include <librnd/rnd_config.h>
 #include <librnd/core/math_helper.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/pixmap.h>
+#include <librnd/hid/pixmap.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,24 +25,23 @@
 #include <librnd/core/plugins.h>
 #include <librnd/core/safe_fs.h>
 
-#include <librnd/core/hid.h>
-#include <librnd/core/hid_nogui.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/hid_nogui.h>
 #include <librnd/core/hid_cfg.h>
 #include "lesstif.h"
-#include <librnd/core/hid_cfg_input.h>
-#include <librnd/core/hid_attrib.h>
-#include <librnd/core/hid_init.h>
-#include <librnd/core/hid_dad.h>
+#include <librnd/hid/hid_cfg_input.h>
+#include <librnd/hid/hid_attrib.h>
+#include <librnd/hid/hid_init.h>
+#include <librnd/hid/hid_dad.h>
 #include <librnd/core/actions.h>
 #include "ltf_stdarg.h"
-#include <librnd/core/grid.h>
+#include <librnd/hid/grid.h>
 #include <librnd/core/misc_util.h>
 #include <librnd/core/compat_misc.h>
-#include <librnd/core/tool.h>
+#include <librnd/hid/tool.h>
 #include <librnd/core/globalconst.h>
 
 #include "wt_preview.h"
-#include "dlg_fileselect.h"
 
 #include "FillBox.h"
 
@@ -54,7 +53,7 @@
 
 const char *lesstif_cookie = "lesstif HID";
 
-rnd_hidlib_t *ltf_hidlib;
+rnd_design_t *ltf_hidlib;
 
 rnd_hid_cfg_mouse_t lesstif_mouse;
 rnd_hid_cfg_keys_t lesstif_keymap;
@@ -93,8 +92,6 @@ XtAppContext app_context;
 Widget appwidget, ltf_fullscreen_left, ltf_fullscreen_top, ltf_fullscreen_bottom;
 Display *display;
 static Window window = 0;
-static int old_cursor_mode = -1;
-static int over_point = 0;
 
 static Pixmap pixmap = 0;          /* Current pixmap we are drawing to (either main_pixmap in direct mode or mask_pixmap when drawing the sketch) */
 static Pixmap main_pixmap = 0;     /* 'Base pixmap', the final output all sketches are composed onto (then drawn) */
@@ -132,7 +129,7 @@ static int view_width, view_height;
 static rnd_coord_t view_left_x = 0, view_top_y = 0;
 /* Denotes PCB units per screen pixel.  Larger numbers mean zooming
    out - the largest value means you are looking at the whole
-   board.  */
+   design.  */
 static double view_zoom = RND_MIL_TO_COORD(10);
 static rnd_bool autofade = 0;
 static rnd_bool crosshair_on = rnd_true;
@@ -254,6 +251,17 @@ static void ltf_dock_leave(rnd_hid_t *hid, rnd_hid_dad_subdialog_t *sub)
 	XtUnmanageChild(frame);
 }
 
+static void ltf_update_topwin_dock_hidlibs(rnd_design_t *new_dsg)
+{
+	int n;
+	for(n = 0; n < RND_HID_DOCK_max; n++) {
+		rnd_hid_dad_subdialog_t *sub;
+		for(sub = gdl_first(&ltf_dock[n]); sub != NULL; sub = gdl_next(&ltf_dock[n], sub)) {
+			docked_t *docked = sub->parent_ctx;
+			lesstif_attr_sub_update_hidlib(docked->hid_ctx, new_dsg);
+		}
+	}
+}
 
 typedef struct {
 	void *hid_ctx;
@@ -324,7 +332,7 @@ static void Pan(int mode, rnd_coord_t x, rnd_coord_t y);
 
 /* Px converts view->design, Vx converts pcb->view */
 
-static inline int Vx(rnd_coord_t x)
+RND_INLINE int Vx(rnd_coord_t x)
 {
 	int rv = (x - view_left_x) / view_zoom + 0.5;
 	if (rnd_conf.editor.view.flip_x)
@@ -332,7 +340,7 @@ static inline int Vx(rnd_coord_t x)
 	return rv;
 }
 
-static inline int Vy(rnd_coord_t y)
+RND_INLINE int Vy(rnd_coord_t y)
 {
 	int rv = (y - view_top_y) / view_zoom + 0.5;
 	if (rnd_conf.editor.view.flip_y)
@@ -340,24 +348,24 @@ static inline int Vy(rnd_coord_t y)
 	return rv;
 }
 
-static inline int Vz(rnd_coord_t z)
+RND_INLINE int Vz(rnd_coord_t z)
 {
 	return z / view_zoom + 0.5;
 }
 
-static inline int Vw(rnd_coord_t w)
+RND_INLINE int Vw(rnd_coord_t w)
 {
 	return w < 0 ? -w : w / view_zoom + 0.5;
 }
 
-static inline rnd_coord_t Px(int x)
+RND_INLINE rnd_coord_t Px(int x)
 {
 	if (rnd_conf.editor.view.flip_x)
 		x = view_width - x;
 	return x * view_zoom + view_left_x;
 }
 
-static inline rnd_coord_t Py(int y)
+RND_INLINE rnd_coord_t Py(int y)
 {
 	if (rnd_conf.editor.view.flip_y)
 		y = view_height - y;
@@ -383,61 +391,36 @@ Pixel lesstif_parse_color(const rnd_color_t *value)
 	return 0;
 }
 
-/* ------------------------------------------------------------ */
-
-/* Called from the core when it's busy doing something and we need to indicate that to the user.  */
-static void ltf_busy(rnd_hid_t *hid, rnd_bool busy)
-{
-	static Cursor busy_cursor = 0;
-	if (!lesstif_active)
-		return;
-
-	if (busy) {
-		if (busy_cursor == 0)
-			busy_cursor = XCreateFontCursor(display, XC_watch);
-		XDefineCursor(display, window, busy_cursor);
-		XFlush(display);
-		old_cursor_mode = -1;
-	}
-	else
-		need_idle_proc(); /* restores the cursor */
-}
-
-
 /* ---------------------------------------------------------------------- */
 
 /* Local actions.  */
 
-static void PointCursor(rnd_hid_t *hid, rnd_bool grabbed)
-{
-	if (grabbed > 0)
-		over_point = 1;
-	else
-		over_point = 0;
-	old_cursor_mode = -1;
-}
+extern void LesstifNetlistChanged(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[]);
+extern void LesstifLibraryChanged(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[]);
 
-extern void LesstifNetlistChanged(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[]);
-extern void LesstifLibraryChanged(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[]);
 
-
-static void ltf_set_hidlib(rnd_hid_t *hid, rnd_hidlib_t *hidlib)
+static void ltf_set_hidlib(rnd_hid_t *hid, rnd_design_t *hidlib)
 {
+	rnd_coord_t siz;
 	ltf_hidlib = hidlib;
+	ltf_update_topwin_dock_hidlibs(hidlib);
 	if ((work_area == 0) || (hidlib == NULL))
 		return;
-	/*rnd_printf("PCB Changed! %$mD\n", ltf_hidlib->size_x, ltf_hidlib->size_y); */
+	/*rnd_printf("PCB Changed! %$mD\n", rnd_dwg_get_size_x(ltf_hidlib), rnd_dwg_get_size_y(ltf_hidlib)); */
+	siz = rnd_dwg_get_size_x(ltf_hidlib);
 	stdarg_n = 0;
-	stdarg(XmNminimum, 0);
-	stdarg(XmNvalue, 0);
-	stdarg(XmNsliderSize, ltf_hidlib->size_x ? ltf_hidlib->size_x : 1);
-	stdarg(XmNmaximum, ltf_hidlib->size_x ? ltf_hidlib->size_x : 1);
+	stdarg(XmNminimum, ltf_hidlib->dwg.X1);
+	stdarg(XmNvalue, ltf_hidlib->dwg.X1);
+	stdarg(XmNsliderSize, siz > 0 ? siz : 1);
+	stdarg(XmNmaximum, ltf_hidlib->dwg.X2 ? ltf_hidlib->dwg.X2 : 1);
 	XtSetValues(hscroll, stdarg_args, stdarg_n);
+
+	siz = rnd_dwg_get_size_y(ltf_hidlib);
 	stdarg_n = 0;
-	stdarg(XmNminimum, 0);
-	stdarg(XmNvalue, 0);
-	stdarg(XmNsliderSize, ltf_hidlib->size_y ? ltf_hidlib->size_y : 1);
-	stdarg(XmNmaximum, ltf_hidlib->size_y ? ltf_hidlib->size_y : 1);
+	stdarg(XmNminimum, ltf_hidlib->dwg.Y1);
+	stdarg(XmNvalue, ltf_hidlib->dwg.Y1);
+	stdarg(XmNsliderSize, siz > 0 ? siz : 1);
+	stdarg(XmNmaximum, ltf_hidlib->dwg.Y2 ? ltf_hidlib->dwg.Y2 : 1);
 	XtSetValues(vscroll, stdarg_args, stdarg_n);
 	zoom_max();
 
@@ -445,11 +428,6 @@ static void ltf_set_hidlib(rnd_hid_t *hid, rnd_hidlib_t *hidlib)
 	return;
 }
 
-static rnd_hidlib_t *ltf_get_hidlib(rnd_hid_t *hid)
-{
-	return ltf_hidlib;
-}
-
 static Widget m_cmd = 0, m_cmd_label;
 static int cmd_is_active = 0;
 
@@ -613,10 +591,11 @@ static double ltf_benchmark(rnd_hid_t *hid)
 	save_main = main_pixmap;
 	main_pixmap = window;
 
-	ctx.view.X1 = 0;
-	ctx.view.Y1 = 0;
-	ctx.view.X2 = ltf_hidlib->size_x;
-	ctx.view.Y2 = ltf_hidlib->size_y;
+	ctx.design = ltf_hidlib;
+	ctx.view.X1 = ltf_hidlib->dwg.X1;
+	ctx.view.Y1 = ltf_hidlib->dwg.Y1;
+	ctx.view.X2 = ltf_hidlib->dwg.X2;
+	ctx.view.Y2 = ltf_hidlib->dwg.X2;
 
 	pixmap = window;
 	XSync(display, 0);
@@ -658,7 +637,7 @@ typedef struct {
 	unsigned flip_y:1;
 } rnd_ltf_pixmap_t;
 
-static void rnd_ltf_draw_pixmap_(rnd_hidlib_t *hidlib, rnd_ltf_pixmap_t *lpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t dw, rnd_coord_t dh)
+static void rnd_ltf_draw_pixmap_(rnd_design_t *hidlib, rnd_ltf_pixmap_t *lpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t dw, rnd_coord_t dh)
 {
 	int w, h, sx3, done = 0;
 
@@ -833,6 +812,10 @@ static void rnd_ltf_draw_pixmap(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy,
 	}
 	if (pixmap->hid_data != NULL) {
 		double rsx, rsy, ca = cos(pixmap->tr_rot / RND_RAD_TO_DEG), sa = sin(pixmap->tr_rot / RND_RAD_TO_DEG);
+
+		if (ca < 0) ca = -ca;
+		if (sa < 0) sa = -sa;
+
 		rsx = (double)sx * ca + (double)sy * sa;
 		rsy = (double)sy * ca + (double)sx * sa;
 		rnd_ltf_draw_pixmap_(ltf_hidlib, pixmap->hid_data, cx - rsx/2, cy - rsy/2, rsx, rsy);
@@ -868,7 +851,7 @@ static void DrawBackgroundImage()
 
 	if (lpm.pxm == NULL)
 		lpm.pxm = &ltf_bg_img;
-	rnd_ltf_draw_pixmap_(ltf_hidlib, &lpm, 0, 0, ltf_hidlib->size_x, ltf_hidlib->size_y);
+	rnd_ltf_draw_pixmap_(ltf_hidlib, &lpm, 0, 0, ltf_hidlib->dwg.X2, ltf_hidlib->dwg.Y2);
 }
 
 static void LoadBackgroundImage(const char *fn)
@@ -879,28 +862,29 @@ static void LoadBackgroundImage(const char *fn)
 
 /* ---------------------------------------------------------------------- */
 
-static const rnd_export_opt_t *lesstif_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *lesstif_get_export_options(rnd_hid_t *hid, int *n, rnd_design_t *dsg, void *appspec)
 {
 	if (n != NULL)
 		*n = sizeof(lesstif_attribute_list) / sizeof(rnd_export_opt_t);
 	return lesstif_attribute_list;
 }
 
-static void set_scroll(Widget s, int pos, int view, int pcb)
+static void set_scroll(Widget s, int pos, int view, int min, int max)
 {
 	unsigned int sz = view * view_zoom;
-	if (sz > pcb)
-		sz = pcb;
-	if (pos > pcb - sz)
-		pos = pcb - sz;
-	if (pos < 0)
-		pos = 0;
+	if (sz > max)
+		sz = max;
+	if (pos > max - sz)
+		pos = max - sz;
+	if (pos < min)
+		pos = min;
 	stdarg_n = 0;
 	stdarg(XmNvalue, pos);
 	stdarg(XmNsliderSize, sz);
 	stdarg(XmNincrement, view_zoom);
 	stdarg(XmNpageIncrement, sz);
-	stdarg(XmNmaximum, pcb);
+	stdarg(XmNminimum, min);
+	stdarg(XmNmaximum, max);
 	XtSetValues(s, stdarg_args, stdarg_n);
 }
 
@@ -908,29 +892,29 @@ void lesstif_pan_fixup()
 {
 	if (ltf_hidlib == NULL)
 		return;
-	if (view_left_x > ltf_hidlib->size_x + (view_width * view_zoom))
-		view_left_x = ltf_hidlib->size_x + (view_width * view_zoom);
-	if (view_top_y > ltf_hidlib->size_y + (view_height * view_zoom))
-		view_top_y = ltf_hidlib->size_y + (view_height * view_zoom);
-	if (view_left_x < -(view_width * view_zoom))
-		view_left_x = -(view_width * view_zoom);
-	if (view_top_y < -(view_height * view_zoom))
-		view_top_y = -(view_height * view_zoom);
+	if (view_left_x > ltf_hidlib->dwg.X2 + (view_width * view_zoom))
+		view_left_x = ltf_hidlib->dwg.X2 + (view_width * view_zoom);
+	if (view_top_y > ltf_hidlib->dwg.Y2 + (view_height * view_zoom))
+		view_top_y = ltf_hidlib->dwg.Y2 + (view_height * view_zoom);
+	if (view_left_x < ltf_hidlib->dwg.X1 - (view_width * view_zoom))
+		view_left_x = ltf_hidlib->dwg.X1 - (view_width * view_zoom);
+	if (view_top_y < ltf_hidlib->dwg.Y1 - (view_height * view_zoom))
+		view_top_y = ltf_hidlib->dwg.Y1 - (view_height * view_zoom);
 
-	set_scroll(hscroll, view_left_x, view_width, ltf_hidlib->size_x);
-	set_scroll(vscroll, view_top_y, view_height, ltf_hidlib->size_y);
+	set_scroll(hscroll, view_left_x, view_width, ltf_hidlib->dwg.X1, ltf_hidlib->dwg.X2);
+	set_scroll(vscroll, view_top_y, view_height, ltf_hidlib->dwg.Y1, ltf_hidlib->dwg.Y2);
 
 	lesstif_invalidate_all(rnd_gui);
 }
 
 static void zoom_max()
 {
-	double new_zoom = ltf_hidlib->size_x / view_width;
-	if (new_zoom < ltf_hidlib->size_y / view_height)
-		new_zoom = ltf_hidlib->size_y / view_height;
+	double new_zoom = rnd_dwg_get_size_x(ltf_hidlib) / view_width;
+	if (new_zoom < rnd_dwg_get_size_y(ltf_hidlib) / view_height)
+		new_zoom = rnd_dwg_get_size_y(ltf_hidlib) / view_height;
 
-	view_left_x = -(view_width * new_zoom - ltf_hidlib->size_x) / 2;
-	view_top_y = -(view_height * new_zoom - ltf_hidlib->size_y) / 2;
+	view_left_x = ltf_hidlib->dwg.X1 - (view_width * new_zoom - rnd_dwg_get_size_x(ltf_hidlib)) / 2;
+	view_top_y = ltf_hidlib->dwg.Y1 - (view_height * new_zoom - rnd_dwg_get_size_y(ltf_hidlib)) / 2;
 	view_zoom = new_zoom;
 	rnd_pixel_slop = view_zoom;
 	lesstif_pan_fixup();
@@ -953,9 +937,9 @@ static void zoom_to(double new_zoom, rnd_coord_t x, rnd_coord_t y)
 	if (rnd_conf.editor.view.flip_y)
 		yfrac = 1 - yfrac;
 
-	max_zoom = ltf_hidlib->size_x / view_width;
-	if (max_zoom < ltf_hidlib->size_y / view_height)
-		max_zoom = ltf_hidlib->size_y / view_height;
+	max_zoom = rnd_dwg_get_size_x(ltf_hidlib) / view_width;
+	if (max_zoom < rnd_dwg_get_size_y(ltf_hidlib) / view_height)
+		max_zoom = rnd_dwg_get_size_y(ltf_hidlib) / view_height;
 
 	max_zoom *= MAX_ZOOM_SCALE;
 
@@ -1012,26 +996,8 @@ static void Pan(int mode, rnd_coord_t x, rnd_coord_t y)
 	static rnd_coord_t opx, opy;
 
 	panning = mode;
-	/* This is for ctrl-pan, where the viewport's position is directly
-	   proportional to the cursor position in the window (like the Xaw
-	   thumb panner) */
-TODO("remove this if there is no bugreport for a long time");
-#if 0
-	if (pan_thumb_mode) {
-		opx = x * ltf_hidlib->size_x / view_width;
-		opy = y * ltf_hidlib->size_y / view_height;
-		if (rnd_conf.editor.view.flip_x)
-			opx = ltf_hidlib->size_x - opx;
-		if (rnd_conf.editor.view.flip_y)
-			opy = ltf_hidlib->size_y - opy;
-		view_left_x = opx - view_width / 2 * view_zoom;
-		view_top_y = opy - view_height / 2 * view_zoom;
-		lesstif_pan_fixup();
-	}
-	else
-#endif
 
-	/* This is the start of a regular pan.  On the first click, we
+	/* On the first click, we
 	   remember the coordinates where we "grabbed" the screen.  */
 	if (mode == 1) {
 		ox = x;
@@ -1072,9 +1038,21 @@ static rnd_hid_cfg_mod_t lesstif_mb2cfg(int but)
 	return 0;
 }
 
+/* save rnd_render and restore at the end: we may get an async expose draw
+   while it is set to an exporter */
+#define SAVE_HID() \
+do { \
+	hid_save = rnd_render; \
+	rnd_render = &lesstif_hid; \
+} while(0)
+
+#define RESTORE_HID() \
+ rnd_render = hid_save
+
 static void work_area_input(Widget w, XtPointer v, XEvent * e, Boolean * ctd)
 {
 	static int pressed_button = 0;
+	rnd_hid_t *hid_save;
 
 	show_crosshair(0);
 	switch (e->type) {
@@ -1160,6 +1138,7 @@ static void work_area_input(Widget w, XtPointer v, XEvent * e, Boolean * ctd)
 		break;
 
 	case LeaveNotify:
+		SAVE_HID();
 		crosshair_in_window = 0;
 		if (crosshair_on && (rnd_app.draw_attached != NULL))
 			rnd_app.draw_attached(ltf_hidlib, 1);
@@ -1167,6 +1146,7 @@ static void work_area_input(Widget w, XtPointer v, XEvent * e, Boolean * ctd)
 			rnd_app.draw_marks(ltf_hidlib, 1);
 		ShowCrosshair(rnd_false);
 		need_idle_proc();
+		RESTORE_HID();
 		break;
 
 	case EnterNotify:
@@ -1494,7 +1474,8 @@ TODO("dock: layersel depends on vertical text");
 	stdarg_n = 0;
 	stdarg(XmNorientation, XmVERTICAL);
 	stdarg(XmNprocessingDirection, XmMAX_ON_BOTTOM);
-	stdarg(XmNmaximum, ltf_hidlib->size_y ? ltf_hidlib->size_y : 1);
+	stdarg(XmNminimum, ltf_hidlib->dwg.Y1 ? ltf_hidlib->dwg.Y1 : 1);
+	stdarg(XmNmaximum, ltf_hidlib->dwg.Y2 ? ltf_hidlib->dwg.Y2 : 1);
 	vscroll = XmCreateScrollBar(mainwind, XmStrCast("vscroll"), stdarg_args, stdarg_n);
 	XtAddCallback(vscroll, XmNvalueChangedCallback, (XtCallbackProc) scroll_callback, (XtPointer) & view_top_y);
 	XtAddCallback(vscroll, XmNdragCallback, (XtCallbackProc) scroll_callback, (XtPointer) & view_top_y);
@@ -1502,7 +1483,8 @@ TODO("dock: layersel depends on vertical text");
 
 	stdarg_n = 0;
 	stdarg(XmNorientation, XmHORIZONTAL);
-	stdarg(XmNmaximum, ltf_hidlib->size_x ? ltf_hidlib->size_x : 1);
+	stdarg(XmNminimum, ltf_hidlib->dwg.X1 ? ltf_hidlib->dwg.X1 : 1);
+	stdarg(XmNmaximum, ltf_hidlib->dwg.X2 ? ltf_hidlib->dwg.X2 : 1);
 	hscroll = XmCreateScrollBar(mainwind, XmStrCast("hscroll"), stdarg_args, stdarg_n);
 	XtAddCallback(hscroll, XmNvalueChangedCallback, (XtCallbackProc) scroll_callback, (XtPointer) & view_left_x);
 	XtAddCallback(hscroll, XmNdragCallback, (XtCallbackProc) scroll_callback, (XtPointer) & view_left_x);
@@ -1568,7 +1550,7 @@ static void ltf_topwin_make_bottom(void)
 	ltf_fullscreen_bottom = messages;
 }
 
-static void lesstif_do_export(rnd_hid_t *hid, rnd_hid_attr_val_t *options)
+static void lesstif_do_export(rnd_hid_t *hid, rnd_design_t *design, rnd_hid_attr_val_t *options, void *appspec)
 {
 	Dimension width, height;
 
@@ -1920,7 +1902,6 @@ static void draw_grid()
 	rnd_coord_t x, y;
 	int n, n3;
 	static GC grid_gc = 0;
-	static const int enable_sparse = 0, min_dist_px = 4;
 
 	if (!rnd_conf.editor.draw_grid)
 		return;
@@ -1930,9 +1911,9 @@ static void draw_grid()
 		grd = 1;
 
 	if (Vz(grd) < RND_MIN_GRID_DISTANCE) {
-		if (!/*rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.sparse*/enable_sparse)
+		if (!rnd_conf.editor.global_grid.sparse)
 			return;
-		grd *= (/*rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.min_dist_px*/ min_dist_px / Vz(grd));
+		grd *= (rnd_conf.editor.global_grid.min_dist_px / Vz(grd));
 	}
 
 	if (!grid_gc) {
@@ -2033,6 +2014,9 @@ static void draw_grid()
 
 static Boolean idle_proc(XtPointer dummy)
 {
+	rnd_hid_t *hid_save;
+
+	SAVE_HID();
 	if (need_redraw) {
 		int mx, my;
 		rnd_hid_expose_ctx_t ctx;
@@ -2040,6 +2024,7 @@ static Boolean idle_proc(XtPointer dummy)
 		pixmap = main_pixmap;
 		mx = view_width;
 		my = view_height;
+		ctx.design = ltf_hidlib;
 		ctx.view.X1 = Px(0);
 		ctx.view.Y1 = Py(0);
 		ctx.view.X2 = Px(view_width);
@@ -2057,13 +2042,13 @@ static Boolean idle_proc(XtPointer dummy)
 		XSetForeground(display, bg_gc, bgcolor);
 		XFillRectangle(display, main_pixmap, bg_gc, 0, 0, mx, my);
 
-		if (ctx.view.X1 < 0 || ctx.view.Y1 < 0 || ctx.view.X2 > ltf_hidlib->size_x || ctx.view.Y2 > ltf_hidlib->size_y) {
+		if (ctx.view.X1 < ltf_hidlib->dwg.X1 || ctx.view.Y1 < ltf_hidlib->dwg.Y1 || ctx.view.X2 > ltf_hidlib->dwg.X2 || ctx.view.Y2 > ltf_hidlib->dwg.Y2) {
 			int leftmost, rightmost, topmost, bottommost;
 
-			leftmost = Vx(0);
-			rightmost = Vx(ltf_hidlib->size_x);
-			topmost = Vy(0);
-			bottommost = Vy(ltf_hidlib->size_y);
+			leftmost = Vx(ltf_hidlib->dwg.X1);
+			rightmost = Vx(ltf_hidlib->dwg.X2);
+			topmost = Vy(ltf_hidlib->dwg.Y1);
+			bottommost = Vy(ltf_hidlib->dwg.Y2);
 			if (leftmost > rightmost) {
 				int t = leftmost;
 				leftmost = rightmost;
@@ -2127,6 +2112,7 @@ TODO(": remove this, update-on should handle all cases")
 
 	show_crosshair(1);
 	idle_proc_set = 0;
+	RESTORE_HID();
 	return True;
 }
 
@@ -2150,7 +2136,7 @@ static void lesstif_invalidate_lr(rnd_hid_t *hid, rnd_coord_t l, rnd_coord_t r,
 void lesstif_invalidate_all(rnd_hid_t *hid)
 {
 	if (ltf_hidlib != NULL)
-		lesstif_invalidate_lr(hid, 0, ltf_hidlib->size_x, 0, ltf_hidlib->size_y);
+		lesstif_invalidate_lr(hid, ltf_hidlib->dwg.X1, ltf_hidlib->dwg.X2, ltf_hidlib->dwg.Y1, ltf_hidlib->dwg.Y2);
 }
 
 static void lesstif_notify_crosshair_change(rnd_hid_t *hid, rnd_bool changes_complete)
@@ -2177,11 +2163,14 @@ static void lesstif_notify_crosshair_change(rnd_hid_t *hid, rnd_bool changes_com
 	}
 
 	if (invalidate_depth == 0 && crosshair_on) {
+		rnd_hid_t *hid_save;
+		SAVE_HID();
 		save_pixmap = pixmap;
 		pixmap = window;
 		if (rnd_app.draw_attached != NULL)
 			rnd_app.draw_attached(ltf_hidlib, 1);
 		pixmap = save_pixmap;
+		RESTORE_HID();
 	}
 
 	if (!changes_complete)
@@ -2219,7 +2208,7 @@ static void lesstif_notify_mark_change(rnd_hid_t *hid, rnd_bool changes_complete
 		invalidate_depth++;
 }
 
-static int lesstif_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const char *purpose, int purpi, rnd_layer_id_t layer, unsigned int flags, int is_empty, rnd_xform_t **xform)
+static int lesstif_set_layer_group(rnd_hid_t *hid, rnd_design_t *design, rnd_layergrp_id_t group, const char *purpose, int purpi, rnd_layer_id_t layer, unsigned int flags, int is_empty, rnd_xform_t **xform)
 {
 	/* accept anything and draw */
 	return 1;
@@ -2645,7 +2634,7 @@ static int lesstif_mod1_is_pressed(rnd_hid_t *hid)
 
 extern int lesstif_get_coords(rnd_hid_t *hid, const char *msg, rnd_coord_t *x, rnd_coord_t *y, int force);
 
-static void lesstif_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int action)
+static void lesstif_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, rnd_set_crosshair_t action)
 {
 	if (crosshair_x != x || crosshair_y != y) {
 		lesstif_show_crosshair(0);
@@ -2664,7 +2653,7 @@ static void lesstif_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y,
 
 	}
 
-	if (action == HID_SC_PAN_VIEWPORT) {
+	if (action == RND_SC_PAN_VIEWPORT) {
 		Window root, child;
 		unsigned int keys_buttons;
 		int pos_x, pos_y, root_x, root_y;
@@ -2678,9 +2667,9 @@ static void lesstif_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y,
 		else
 			view_top_y = y - pos_y * view_zoom;
 		lesstif_pan_fixup();
-		action = HID_SC_WARP_POINTER;
+		action = RND_SC_WARP_POINTER;
 	}
-	if (action == HID_SC_WARP_POINTER) {
+	if (action == RND_SC_WARP_POINTER) {
 		in_move_event++;
 		XWarpPointer(display, None, window, 0, 0, 0, 0, Vx(x), Vy(y));
 		in_move_event--;
@@ -2788,7 +2777,7 @@ rnd_hidval_t lesstif_watch_file(rnd_hid_t *hid, int fd, unsigned int condition,
 	return ret;
 }
 
-extern void *lesstif_attr_dlg_new(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs_, int n_attrs_, const char *title_, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny);
+extern void lesstif_attr_dlg_new(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs_, int n_attrs_, const char *title_, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny, void **hid_ctx_out);
 
 extern int lesstif_attr_dlg_run(void *hid_ctx);
 extern void lesstif_attr_dlg_raise(void *hid_ctx);
@@ -2803,12 +2792,6 @@ extern void lesstif_attr_dlg_set_help(void *hid_ctx, int idx, const char *val);
 
 #include "wt_preview.c"
 
-static void lesstif_beep(rnd_hid_t *hid)
-{
-	putchar(7);
-	fflush(stdout);
-}
-
 static int lesstif_usage(rnd_hid_t *hid, const char *topic)
 {
 	fprintf(stderr, "\nLesstif GUI command line arguments:\n\n");
@@ -2817,7 +2800,7 @@ static int lesstif_usage(rnd_hid_t *hid, const char *topic)
 	return 0;
 }
 
-static void lesstif_globconf_change_post(rnd_conf_native_t *cfg, int arr_idx)
+static void lesstif_globconf_change_post(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	if (!lesstif_active)
 		return;
@@ -2830,7 +2813,7 @@ static void lesstif_globconf_change_post(rnd_conf_native_t *cfg, int arr_idx)
 	}
 }
 
-static void ltf_confchg_fullscreen(rnd_conf_native_t *cfg, int arr_idx)
+static void ltf_confchg_fullscreen(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	if (!lesstif_active)
 		return;
@@ -2868,7 +2851,7 @@ static void lesstif_conf_regs(const char *cookie)
 
 #include <Xm/CutPaste.h>
 
-static int ltf_clip_set(rnd_hid_t *hid, rnd_hid_clipfmt_t format, const void *data, size_t len)
+static int ltf_clip_set(rnd_hid_t *hid, const char *str)
 {
 	static long cnt = 0;
 	long item_id, data_id;
@@ -2880,7 +2863,7 @@ static int ltf_clip_set(rnd_hid_t *hid, rnd_hid_clipfmt_t format, const void *da
 		return -1;
 	}
 	XmStringFree(lab);
-	if (XmClipboardCopy(display, window, item_id, "STRING", (void *)data, len, ++cnt, &data_id) != XmClipboardSuccess) {
+	if (XmClipboardCopy(display, window, item_id, "STRING", (void *)str, strlen(str), ++cnt, &data_id) != XmClipboardSuccess) {
 		XmClipboardCancelCopy(display, window, item_id);
 		return -1;
 	}
@@ -2891,19 +2874,16 @@ static int ltf_clip_set(rnd_hid_t *hid, rnd_hid_clipfmt_t format, const void *da
 	return 0;
 }
 
-static int ltf_clip_get(rnd_hid_t *hid, rnd_hid_clipfmt_t *format, void **data, size_t *len)
+static char *ltf_clip_get(rnd_hid_t *hid)
 {
 	int res;
-	gds_t tmp;
+	gds_t tmp = {0};
 	char buff[65536];
 	long unsigned bl = 0;
 	long dummy;
 
 	if (XmClipboardStartRetrieve(display, window, CurrentTime) != XmClipboardSuccess)
-		return -1;
-
-	gds_init(&tmp);
-
+		return NULL;
 
 	res = XmClipboardRetrieve(display, window, "STRING", buff, sizeof(buff), &bl, &dummy);
 	if (res == XmClipboardSuccess) {
@@ -2912,18 +2892,7 @@ static int ltf_clip_get(rnd_hid_t *hid, rnd_hid_clipfmt_t *format, void **data,
 	}
 
 	XmClipboardEndRetrieve(display, window);
-	if (tmp.used == 0) {
-		gds_uninit(&tmp);
-		return -1;
-	}
-	*data = tmp.array;
-	*len = tmp.used;
-	return 0;
-}
-
-static void ltf_clip_free(rnd_hid_t *hid, rnd_hid_clipfmt_t format, void *data, size_t len)
-{
-	free(data);
+	return tmp.array;
 }
 
 static void ltf_zoom_win(rnd_hid_t *hid, rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2, rnd_bool set_crosshair)
@@ -3022,7 +2991,6 @@ int pplg_init_hid_lesstif(void)
 	lesstif_hid.name = "lesstif";
 	lesstif_hid.description = "LessTif - a Motif clone for X/Unix";
 	lesstif_hid.gui = 1;
-	lesstif_hid.heavy_term_layer_ind = 1;
 
 	lesstif_hid.get_export_options = lesstif_get_export_options;
 	lesstif_hid.do_export = lesstif_do_export;
@@ -3062,7 +3030,6 @@ int pplg_init_hid_lesstif(void)
 	lesstif_hid.unwatch_file = lesstif_unwatch_file;
 	lesstif_hid.benchmark = ltf_benchmark;
 
-	lesstif_hid.fileselect = rnd_ltf_fileselect;
 	lesstif_hid.attr_dlg_new = lesstif_attr_dlg_new;
 	lesstif_hid.attr_dlg_run = lesstif_attr_dlg_run;
 	lesstif_hid.attr_dlg_raise = lesstif_attr_dlg_raise;
@@ -3075,12 +3042,9 @@ int pplg_init_hid_lesstif(void)
 	lesstif_hid.attr_dlg_set_help = lesstif_attr_dlg_set_help;
 	lesstif_hid.supports_dad_text_markup = 0;
 
-	lesstif_hid.beep = lesstif_beep;
-	lesstif_hid.point_cursor = PointCursor;
 	lesstif_hid.command_entry = lesstif_command_entry;
 	lesstif_hid.clip_set = ltf_clip_set;
 	lesstif_hid.clip_get = ltf_clip_get;
-	lesstif_hid.clip_free = ltf_clip_free;
 
 	lesstif_hid.create_menu_by_node = lesstif_create_menu_widget;
 	lesstif_hid.remove_menu_node = lesstif_remove_menu_node;
@@ -3101,17 +3065,15 @@ int pplg_init_hid_lesstif(void)
 	lesstif_hid.set_top_title = ltf_set_top_title;
 	lesstif_hid.dock_enter = ltf_dock_enter;
 	lesstif_hid.dock_leave = ltf_dock_leave;
-	lesstif_hid.busy = ltf_busy;
 
 	lesstif_hid.draw_pixmap = rnd_ltf_draw_pixmap;
 	lesstif_hid.uninit_pixmap = rnd_ltf_uninit_pixmap;
 
-	lesstif_hid.set_hidlib = ltf_set_hidlib;
-	lesstif_hid.get_hidlib = ltf_get_hidlib;
+	lesstif_hid.set_design = ltf_set_hidlib;
 
 	lesstif_hid.usage = lesstif_usage;
 
-	lesstif_hid.get_dad_hidlib = ltf_attr_get_dad_hidlib;
+	lesstif_hid.get_dad_design = ltf_attr_get_dad_hidlib;
 
 	lesstif_hid.argument_array = ltf_values;
 
