@@ -19,14 +19,14 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "wt_preview.h"
 #include "dialogs.h"
-#include <librnd/core/hid_attrib.h>
+#include <librnd/hid/hid_attrib.h>
 
 static gdl_list_t ltf_previews;
 
@@ -188,7 +188,7 @@ void rnd_ltf_preview_invalidate(const rnd_box_t *screen)
 	rnd_ltf_preview_t *prv;
 
 	for(prv = gdl_first(&ltf_previews); prv != NULL; prv = prv->link.next) {
-		if (!prv->redraw_with_board) continue;
+		if (!prv->redraw_with_design) continue;
 		if (screen != NULL) {
 			rnd_box_t pb;
 			pb.X1 = prv->x1;
