@@ -21,8 +21,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
@@ -31,14 +31,14 @@
    then got reworked heavily by Tibor 'Igor2' Palinkas for pcb-rnd DAD */
 
 #include "config.h"
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 #include "dlg_attribute.h"
 #include <stdlib.h>
 
 #include <librnd/core/rnd_printf.h>
-#include <librnd/core/hid_attrib.h>
-#include <librnd/core/hid_dad_tree.h>
-#include <librnd/core/hid_init.h>
+#include <librnd/hid/hid_attrib.h>
+#include <librnd/hid/hid_dad_tree.h>
+#include <librnd/hid/hid_init.h>
 #include <librnd/core/misc_util.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/event.h>
@@ -51,7 +51,7 @@
 typedef struct attr_dlg_s {
 	void *caller_data; /* WARNING: for now, this must be the first field (see core spinbox enter_cb) */
 	rnd_gtk_t *gctx;
-	rnd_hidlib_t *hidlib; /* active hidlib at the moment the dialog box was created */
+	rnd_design_t *hidlib; /* active hidlib at the moment the dialog box was created */
 	rnd_hid_attribute_t *attrs;
 	GtkWidget **wl;     /* content widget */
 	GtkWidget **wltop;  /* the parent widget, which is different from wl if reparenting (extra boxes, e.g. for framing or scrolling) was needed */
@@ -496,8 +496,7 @@ static int rnd_gtk_attr_dlg_add(attr_dlg_t *ctx, GtkWidget *real_parent, rnd_gtk
 					p = (rnd_gtk_preview_t *)ctx->wl[j];
 					p->flip_local = !!(ctx->attrs[j].rnd_hatt_flags & RND_HATF_PRV_LFLIP);
 					p->flip_global = !!(ctx->attrs[j].rnd_hatt_flags & RND_HATF_PRV_GFLIP);
-					p->view.local_flip = (p->flip_local && !p->flip_global);
-					
+					p->view.local_flip = !p->flip_global; /* if not local, it's hardwired 0;0 which is easiest done by saying it's local and keeping local flips 0 */
 				}
 				break;
 
@@ -862,7 +861,7 @@ void rnd_gtk_attr_dlg_mapped_cb(GObject *self, gpointer user_data)
 	ctx->mapped = 1;
 }
 
-void *rnd_gtk_attr_dlg_new(rnd_hid_t *hid, rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny)
+void rnd_gtk_attr_dlg_new(rnd_hid_t *hid, rnd_gtk_t *gctx, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny, void **hid_ctx_out)
 {
 	GtkWidget *main_vbox;
 	attr_dlg_t *ctx;
@@ -873,6 +872,7 @@ void *rnd_gtk_attr_dlg_new(rnd_hid_t *hid, rnd_gtk_t *gctx, const char *id, rnd_
 	plc[3] = defy;
 
 	ctx = calloc(sizeof(attr_dlg_t), 1);
+	*hid_ctx_out = ctx;
 	ctx->gctx = gctx;
 	ctx->hidlib = gctx->hidlib;
 	ctx->attrs = attrs;
@@ -932,9 +932,6 @@ void *rnd_gtk_attr_dlg_new(rnd_hid_t *hid, rnd_gtk_t *gctx, const char *id, rnd_
 
 	if (rnd_gtk_conf_hid.plugins.hid_gtk.dialog.auto_present)
 		gtk_window_present(GTK_WINDOW(ctx->dialog));
-
-
-	return ctx;
 }
 
 void *rnd_gtk_attr_sub_new(rnd_gtk_t *gctx, GtkWidget *parent_box, rnd_hid_attribute_t *attrs, int n_attrs, void *caller_data)
@@ -962,6 +959,14 @@ void *rnd_gtk_attr_sub_new(rnd_gtk_t *gctx, GtkWidget *parent_box, rnd_hid_attri
 	return ctx;
 }
 
+void rnd_gtk_attr_sub_update_hidlib(void *hid_ctx, rnd_design_t *new_dsg)
+{
+	attr_dlg_t *ctx = hid_ctx;
+
+	ctx->hidlib = new_dsg;
+}
+
+
 int rnd_gtk_attr_dlg_run(void *hid_ctx)
 {
 	attr_dlg_t *ctx = hid_ctx;
@@ -1137,7 +1142,7 @@ void rnd_gtk_dad_fixcolor(void *hid_ctx, const rnd_gtk_color_t *color)
 	}
 }
 
-int rnd_gtk_winplace_cfg(rnd_hidlib_t *hidlib, GtkWidget *widget, void *ctx, const char *id)
+int rnd_gtk_winplace_cfg(rnd_design_t *hidlib, GtkWidget *widget, void *ctx, const char *id)
 {
 	GtkAllocation allocation;
 
@@ -1152,11 +1157,9 @@ int rnd_gtk_winplace_cfg(rnd_hidlib_t *hidlib, GtkWidget *widget, void *ctx, con
 	return 0;
 }
 
-rnd_hidlib_t *rnd_gtk_attr_get_dad_hidlib(void *hid_ctx)
+rnd_design_t *rnd_gtk_attr_get_dad_hidlib(void *hid_ctx)
 {
 	attr_dlg_t *ctx = hid_ctx;
 
-	if (rnd_hid_enable_per_dialog_hidlib)
-		return ctx->hidlib;
-	return ctx->gctx->hidlib;
+	return ctx->hidlib;
 }
