@@ -21,14 +21,14 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
 #include "config.h"
-#include <librnd/core/pixmap.h>
+#include <librnd/hid/pixmap.h>
 
 #include "glue_common.h"
 
@@ -40,6 +40,8 @@
 #include "in_keyboard.h"
 #include "wt_preview.h"
 #include <librnd/core/safe_fs.h>
+#include <librnd/core/rnd_conf.h>
+#include <librnd/plugins/lib_hid_common/lib_hid_common.h>
 
 rnd_gtk_t _ghidgui, *ghidgui = &_ghidgui;
 
@@ -47,7 +49,7 @@ rnd_gtk_t _ghidgui, *ghidgui = &_ghidgui;
 
 #ifdef __WIN32__
 
-#include <librnd/core/hid_init.h>
+#include <librnd/hid/hid_init.h>
 #include <librnd/core/compat_fs.h>
 
 static void rnd_gtkg_win32_init(void)
@@ -79,30 +81,30 @@ static void rnd_gtkg_win32_init(void) {} /* no-op on non-win32 */
 
 static const char *cookie_menu = "gtk hid menu";
 
-static void rnd_gtk_confchg_fullscreen(rnd_conf_native_t *cfg, int arr_idx)
+static void rnd_gtk_confchg_fullscreen(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	if (ghidgui->hid_active)
 		rnd_gtk_fullscreen_apply(&ghidgui->topwin);
 }
 
 
-void rnd_gtk_confchg_checkbox(rnd_conf_native_t *cfg, int arr_idx)
+void rnd_gtk_confchg_checkbox(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	if ((ghidgui->hid_active) && (ghidgui->hidlib != NULL))
 		rnd_gtk_update_toggle_flags(ghidgui->hidlib, &ghidgui->topwin, NULL);
 }
 
-static void rnd_gtk_confchg_cli(rnd_conf_native_t *cfg, int arr_idx)
+static void rnd_gtk_confchg_cli(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	rnd_gtk_command_update_prompt(&ghidgui->topwin.cmd);
 }
 
-static void rnd_gtk_confchg_flip(rnd_conf_native_t *cfg, int arr_idx)
+static void rnd_gtk_confchg_flip(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	rnd_gtk_previews_flip(ghidgui);
 }
 
-static void rnd_gtk_confchg_spec_color(rnd_conf_native_t *cfg, int arr_idx)
+static void rnd_gtk_confchg_spec_color(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	if (!ghidgui->hid_active)
 		return;
@@ -113,7 +115,7 @@ static void rnd_gtk_confchg_spec_color(rnd_conf_native_t *cfg, int arr_idx)
 
 
 
-static void init_conf_watch(rnd_conf_hid_callbacks_t *cbs, const char *path, void (*func)(rnd_conf_native_t *, int))
+static void init_conf_watch(rnd_conf_hid_callbacks_t *cbs, const char *path, void (*func)(rnd_conf_native_t *, int, void *))
 {
 	rnd_conf_native_t *n = rnd_conf_get_field(path);
 	if (n != NULL) {
@@ -149,7 +151,7 @@ static void rnd_gtk_conf_regs(const char *cookie)
 /*** drawing widget - output ***/
 
 /* Do scrollbar scaling based on current port drawing area size and
-   overall PCB board size. */
+   overall design size. */
 void rnd_gtk_tw_ranges_scale(rnd_gtk_t *gctx)
 {
 	rnd_gtk_topwin_t *tw = &gctx->topwin;
@@ -159,8 +161,15 @@ void rnd_gtk_tw_ranges_scale(rnd_gtk_t *gctx)
 	   size in pixels to PCB units and that will be the page size for the Gtk adjustment. */
 	rnd_gtk_zoom_post(view);
 
-	gtkc_scb_zoom_adjustment(tw->h_range, view->width, gctx->hidlib->size_x);
-	gtkc_scb_zoom_adjustment(tw->v_range, view->height, gctx->hidlib->size_y);
+	if (rnd_conf.editor.view.flip_x)
+		gtkc_scb_zoom_adjustment(tw->h_range, view->width, 0, gctx->hidlib->dwg.X2-gctx->hidlib->dwg.X1);
+	else
+		gtkc_scb_zoom_adjustment(tw->h_range, view->width, gctx->hidlib->dwg.X1, gctx->hidlib->dwg.X2+gctx->hidlib->dwg.X1);
+
+	if (rnd_conf.editor.view.flip_y)
+		gtkc_scb_zoom_adjustment(tw->v_range, view->height, 0, gctx->hidlib->dwg.Y2-gctx->hidlib->dwg.Y1);
+	else
+		gtkc_scb_zoom_adjustment(tw->v_range, view->height, gctx->hidlib->dwg.Y1, gctx->hidlib->dwg.Y2+gctx->hidlib->dwg.Y1);
 }
 
 void rnd_gtk_port_ranges_changed(void)
