@@ -5,16 +5,17 @@
 
 #include "rnd_gtk.h"
 #include <librnd/core/actions.h>
+#include <librnd/core/conf_multi.h>
 #include "glue_hid.h"
-#include <librnd/core/hid_nogui.h>
-#include <librnd/core/hid_attrib.h>
-#include <librnd/core/pixmap.h>
+#include <librnd/hid/hid_menu.h>
+#include <librnd/hid/hid_nogui.h>
+#include <librnd/hid/hid_attrib.h>
+#include <librnd/hid/pixmap.h>
 #include "coord_conv.h"
 
 #include "in_keyboard.h"
 #include "bu_dwg_tooltip.h"
 #include "ui_crosshair.h"
-#include "dlg_fileselect.h"
 #include "dlg_attribute.h"
 #include "util_listener.h"
 #include "util_timer.h"
@@ -96,10 +97,13 @@ static void rnd_gtkg_gui_inited(rnd_gtk_t *gctx, int main, int conf)
 	if (conf) ic = 1;
 
 	if (im && ic && first) {
+		rnd_coord_t midx = (gctx->hidlib->dwg.X1 + gctx->hidlib->dwg.X2) / 2;
+		rnd_coord_t midy = (gctx->hidlib->dwg.Y1 + gctx->hidlib->dwg.Y2) / 2;
 		first = 0;
 		rnd_hid_announce_gui_init(gctx->hidlib);
-		rnd_gtk_zoom_view_win(&gctx->port.view, 0, 0, gctx->hidlib->size_x, gctx->hidlib->size_y, 0);
-		rnd_gtk_pan_view_abs(&gctx->port.view, gctx->hidlib->size_x/2, gctx->hidlib->size_y/2, gctx->port.view.canvas_width/2.0, gctx->port.view.canvas_height/2.0);
+		rnd_gtk_zoom_view_win(&gctx->port.view, gctx->hidlib->dwg.X1, gctx->hidlib->dwg.Y1, gctx->hidlib->dwg.X2, gctx->hidlib->dwg.Y2, 0);
+		rnd_gtk_pan_view_abs(&gctx->port.view, midx, midy, gctx->port.view.canvas_width/2.0, gctx->port.view.canvas_height/2.0);
+		gctx->port.view.inited = 1;
 	}
 }
 
@@ -168,7 +172,7 @@ static void rnd_gtkg_main_export_uninit(rnd_gtk_t *gctx, rnd_hid_t *hid)
 	hid->hid_data = NULL;
 }
 
-static void rnd_gtkg_do_export(rnd_hid_t *hid, rnd_hid_attr_val_t *options);
+static void rnd_gtkg_do_export(rnd_hid_t *hid, rnd_design_t *design, rnd_hid_attr_val_t *options, void *appspec);
 
 static void rnd_gtkg_do_exit_(rnd_gtk_t *gctx)
 {
@@ -184,7 +188,7 @@ static void rnd_gtkg_do_exit_(rnd_gtk_t *gctx)
 static void rnd_gtkg_do_exit(rnd_hid_t *hid);
 
 
-void rnd_gtk_topwinplace(rnd_hidlib_t *hidlib, GtkWidget *dialog, const char *id)
+void rnd_gtk_topwinplace(rnd_design_t *hidlib, GtkWidget *dialog, const char *id)
 {
 	int plc[4] = {-1, -1, -1, -1};
 
@@ -247,7 +251,7 @@ static void rnd_gtk_parse_arguments_first(rnd_gtk_t *gctx, rnd_hid_t *hid, int *
 /* Create top level window for routines that will need top_window before rnd_gtk_create_topwin_widgets() is called. */
 int rnd_gtk_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv);
 
-static void rnd_gtkg_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int action)
+static void rnd_gtkg_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, rnd_set_crosshair_t action)
 {
 	rnd_gtk_t *gctx = hid->hid_data;
 	int offset_x, offset_y;
@@ -277,26 +281,9 @@ static rnd_hidval_t rnd_gtkg_watch_file(rnd_hid_t *hid, int fd, unsigned int con
 	return rnd_gtk_watch_file((rnd_gtk_t *)hid->hid_data, fd, condition, func, user_data);
 }
 
-static char *rnd_gtkg_fileselect(rnd_hid_t *hid, const char *title, const char *descr, const char *default_file, const char *default_ext, const rnd_hid_fsd_filter_t *flt, const char *history_tag, rnd_hid_fsd_flags_t flags, rnd_hid_dad_subdialog_t *sub)
+static void rnd_gtk_attr_dlg_new_(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny, void **hid_ctx_out)
 {
-	return rnd_gtk_fileselect(hid, (rnd_gtk_t *)hid->hid_data, title, descr, default_file, default_ext, flt, history_tag, flags, sub);
-}
-
-static void *rnd_gtk_attr_dlg_new_(rnd_hid_t *hid, const char *id, rnd_hid_attribute_t *attrs, int n_attrs, const char *title, void *caller_data, rnd_bool modal, void (*button_cb)(void *caller_data, rnd_hid_attr_ev_t ev), int defx, int defy, int minx, int miny)
-{
-	return rnd_gtk_attr_dlg_new(hid, (rnd_gtk_t *)hid->hid_data, id, attrs, n_attrs, title, caller_data, modal, button_cb, defx, defy, minx, miny);
-}
-
-static void rnd_gtkg_beep(rnd_hid_t *hid);
-
-static void PointCursor(rnd_hid_t *hid, rnd_bool grabbed)
-{
-	rnd_gtk_t *gctx = hid->hid_data;
-
-	if (gctx == NULL)
-		return;
-
-	rnd_gtk_point_cursor(gctx, grabbed);
+	return rnd_gtk_attr_dlg_new(hid, (rnd_gtk_t *)hid->hid_data, id, attrs, n_attrs, title, caller_data, modal, button_cb, defx, defy, minx, miny, hid_ctx_out);
 }
 
 static int rnd_gtk_remove_menu_node(rnd_hid_t *hid, lht_node_t *node)
@@ -345,36 +332,18 @@ static const char *rnd_gtk_command_entry(rnd_hid_t *hid, const char *ovr, int *c
 	return rnd_gtk_cmd_command_entry(&gctx->topwin.cmd, ovr, cursor);
 }
 
-static int rnd_gtkg_clip_set(rnd_hid_t *hid, rnd_hid_clipfmt_t format, const void *data, size_t len)
+static int rnd_gtkg_clip_set(rnd_hid_t *hid, const char *str)
 {
 	rnd_gtk_t *gctx = hid->hid_data;
-
-	switch(format) {
-		case RND_HID_CLIPFMT_TEXT:
-			gtkc_clipboard_set_text(gctx->topwin.drawing_area, data);
-			break;
-	}
+	gtkc_clipboard_set_text(gctx->topwin.drawing_area, str);
 	return 0;
 }
 
-int rnd_gtkg_clip_get(rnd_hid_t *hid, rnd_hid_clipfmt_t *format, void **data, size_t *len)
+char *rnd_gtkg_clip_get(rnd_hid_t *hid)
 {
 	rnd_gtk_t *gctx = hid->hid_data;
-	int res = gtkc_clipboard_get_text(gctx->topwin.drawing_area, data, len);
-
-	if (res == 0)
-		*format = RND_HID_CLIPFMT_TEXT;
-
-	return res;
-}
-
-void rnd_gtkg_clip_free(rnd_hid_t *hid, rnd_hid_clipfmt_t format, void *data, size_t len)
-{
-	switch(format) {
-		case RND_HID_CLIPFMT_TEXT:
-			g_free(data);
-			break;
-	}
+	char *data = gtkc_clipboard_get_text(gctx->topwin.drawing_area);
+	return (data == NULL) ? NULL : rnd_strdup(data);
 }
 
 static void rnd_gtkg_iterate(rnd_hid_t *hid);
@@ -460,7 +429,7 @@ static int rnd_gtkg_open_popup(rnd_hid_t *hid, const char *menupath)
 	return 0;
 }
 
-static void rnd_gtkg_set_hidlib(rnd_hid_t *hid, rnd_hidlib_t *hidlib)
+static void rnd_gtkg_set_hidlib(rnd_hid_t *hid, rnd_design_t *hidlib)
 {
 	rnd_gtk_t *gctx = hid->hid_data;
 
@@ -468,6 +437,7 @@ static void rnd_gtkg_set_hidlib(rnd_hid_t *hid, rnd_hidlib_t *hidlib)
 		return;
 
 	gctx->hidlib = hidlib;
+	rnd_gtk_tw_update_dock_hidlib(&gctx->topwin, hidlib);
 
 	if(!gctx->hid_active)
 		return;
@@ -478,18 +448,16 @@ static void rnd_gtkg_set_hidlib(rnd_hid_t *hid, rnd_hidlib_t *hidlib)
 	if (!gctx->port.drawing_allowed)
 		return;
 
+	if (ghidgui->impl.set_special_colors != NULL)
+		ghidgui->impl.set_special_colors(NULL);
+
 	rnd_gtk_tw_ranges_scale(gctx);
-	rnd_gtk_zoom_view_win(&gctx->port.view, 0, 0, hidlib->size_x, hidlib->size_y, 0);
-}
-
-static rnd_hidlib_t *rnd_gtkg_get_hidlib(rnd_hid_t *hid)
-{
-	rnd_gtk_t *gctx = hid->hid_data;
-
-	if (gctx == NULL)
-		return NULL;
-
-	return gctx->hidlib;
+	if (!gctx->port.view.inited) {
+		rnd_gtk_zoom_view_win(&gctx->port.view, hidlib->dwg.X1, hidlib->dwg.Y1, hidlib->dwg.X2, hidlib->dwg.Y2, 0);
+		gctx->port.view.inited = 1;
+	}
+	else
+		rnd_gui->invalidate_all(rnd_gui); /* switched design */
 }
 
 static void rnd_gtkg_reg_mouse_cursor(rnd_hid_t *hid, int idx, const char *name, const unsigned char *pixel, const unsigned char *mask)
@@ -508,17 +476,6 @@ static void rnd_gtkg_set_top_title(rnd_hid_t *hid, const char *title)
 	rnd_gtk_tw_set_title(&gctx->topwin, title);
 }
 
-static void rnd_gtkg_busy(rnd_hid_t *hid, rnd_bool busy)
-{
-	rnd_gtk_t *gctx = hid->hid_data;
-	if ((gctx == NULL) || (!gctx->hid_active))
-		return;
-	if (busy)
-		rnd_gtk_watch_cursor(gctx);
-	else
-		rnd_gtk_restore_cursor(gctx);
-}
-
 static int rnd_gtkg_shift_is_pressed(rnd_hid_t *hid)
 {
 	rnd_gtk_t *gctx = hid->hid_data;
@@ -597,6 +554,10 @@ static void rnd_gtkg_draw_pixmap(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy,
 		rnd_gtkg_init_pixmap(hid, pixmap);
 	if (pixmap->hid_data != NULL) {
 		double rsx, rsy, ca = cos(pixmap->tr_rot / RND_RAD_TO_DEG), sa = sin(pixmap->tr_rot / RND_RAD_TO_DEG);
+
+		if (ca < 0) ca = -ca;
+		if (sa < 0) sa = -sa;
+
 		rsx = (double)sx * ca + (double)sy * sa;
 		rsy = (double)sy * ca + (double)sx * sa;
 /*rnd_trace("GUI scale: %mm %mm -> %mm %mm\n", sx, sy, (rnd_coord_t)rsx, (rnd_coord_t)rsy);*/
@@ -604,12 +565,15 @@ static void rnd_gtkg_draw_pixmap(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy,
 	}
 }
 
+static const char gtk_hid_cookie[] = "gtk hid";
+
 static void rnd_gtkg_uninit_pixmap(rnd_hid_t *hid, rnd_pixmap_t *pixmap)
 {
 	if (pixmap->hid_data != NULL) {
 		rnd_gtkg_uninit_pixmap_(hid, pixmap);
 		pixmap->hid_data = NULL;
 	}
+	rnd_conf_state_plug_unreg_all_cookie(gtk_hid_cookie);
 }
 
 void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
@@ -620,7 +584,6 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 
 	dst->struct_size = sizeof(rnd_hid_t);
 	dst->gui = 1;
-	dst->heavy_term_layer_ind = 1;
 	dst->allow_dad_before_init = 1;
 
 	dst->do_export = rnd_gtkg_do_export;
@@ -638,7 +601,6 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 	dst->watch_file = rnd_gtkg_watch_file;
 	dst->unwatch_file = rnd_gtk_unwatch_file;
 
-	dst->fileselect = rnd_gtkg_fileselect;
 	dst->attr_dlg_new = rnd_gtk_attr_dlg_new_;
 	dst->attr_dlg_run = rnd_gtk_attr_dlg_run;
 	dst->attr_dlg_raise = rnd_gtk_attr_dlg_raise;
@@ -656,8 +618,6 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 	dst->dock_enter = rnd_gtkg_dock_enter;
 	dst->dock_leave = rnd_gtkg_dock_leave;
 
-	dst->beep = rnd_gtkg_beep;
-	dst->point_cursor = PointCursor;
 	dst->benchmark = rnd_gtkg_benchmark;
 
 	dst->command_entry = rnd_gtk_command_entry;
@@ -669,7 +629,6 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 
 	dst->clip_set  = rnd_gtkg_clip_set;
 	dst->clip_get  = rnd_gtkg_clip_get;
-	dst->clip_free = rnd_gtkg_clip_free;
 
 	dst->zoom_win = rnd_gtkg_zoom_win;
 	dst->zoom = rnd_gtkg_zoom;
@@ -681,11 +640,9 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 	dst->reg_mouse_cursor = rnd_gtkg_reg_mouse_cursor;
 	dst->set_mouse_cursor = rnd_gtkg_set_mouse_cursor;
 	dst->set_top_title = rnd_gtkg_set_top_title;
-	dst->busy = rnd_gtkg_busy;
 
-	dst->set_hidlib = rnd_gtkg_set_hidlib;
-	dst->get_hidlib = rnd_gtkg_get_hidlib;
-	dst->get_dad_hidlib = rnd_gtk_attr_get_dad_hidlib;
+	dst->set_design = rnd_gtkg_set_hidlib;
+	dst->get_dad_design = rnd_gtk_attr_get_dad_hidlib;
 
 	dst->key_state = &rnd_gtk_keymap;
 
@@ -695,4 +652,8 @@ void rnd_gtk_glue_hid_init(rnd_hid_t *dst)
 	dst->uninit_pixmap = rnd_gtkg_uninit_pixmap;
 
 	dst->hid_data = ghidgui;
+
+	/* save/load only the first few fields that are the zoom states */
+	rnd_conf_state_plug_reg(&ghidgui->port.view, (char *)&ghidgui->port.view.ctx - (char *)&ghidgui->port.view, gtk_hid_cookie);
+
 }
