@@ -3,9 +3,9 @@
 #include <stdio.h>
 
 #include <librnd/core/color_cache.h>
-#include <librnd/core/hid_attrib.h>
-#include <librnd/core/hidlib_conf.h>
-#include <librnd/core/pixmap.h>
+#include <librnd/hid/hid_attrib.h>
+#include <librnd/core/rnd_conf.h>
+#include <librnd/hid/pixmap.h>
 #include <librnd/core/globalconst.h>
 
 #include <librnd/plugins/lib_hid_common/clip.h>
@@ -64,14 +64,14 @@ void ghid_gl_render_burst(rnd_hid_t *hid, rnd_burst_op_t op, const rnd_box_t *sc
 	rnd_gui->coord_per_pix = ghidgui->port.view.coord_per_px;
 }
 
-int ghid_gl_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const char *purpose, int purpi, rnd_layer_id_t layer, unsigned int flags, int is_empty, rnd_xform_t **xform)
+int ghid_gl_set_layer_group(rnd_hid_t *hid, rnd_design_t *design, rnd_layergrp_id_t group, const char *purpose, int purpi, rnd_layer_id_t layer, unsigned int flags, int is_empty, rnd_xform_t **xform)
 {
-	rnd_hidlib_t *hidlib = ghidgui->hidlib;
+	rnd_design_t *hidlib = ghidgui->hidlib;
 	render_priv_t *priv = ghidgui->port.render_priv;
 	double tx, ty, zx, zy, zz;
 
-	tx = rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->size_x : -ghidgui->port.view.x0;
-	ty = rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->size_y : -ghidgui->port.view.y0;
+	tx = rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->dwg.X2 : -ghidgui->port.view.x0;
+	ty = rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->dwg.Y2 : -ghidgui->port.view.y0;
 	zx = (rnd_conf.editor.view.flip_x ? -1. : 1.) / ghidgui->port.view.coord_per_px;
 	zy = (rnd_conf.editor.view.flip_y ? -1. : 1.) / ghidgui->port.view.coord_per_px;
 	zz = ((rnd_conf.editor.view.flip_x == rnd_conf.editor.view.flip_y) ? 1. : -1.) / ghidgui->port.view.coord_per_px;
@@ -108,7 +108,7 @@ rnd_hid_gc_t ghid_gl_make_gc(rnd_hid_t *hid)
 	return rv;
 }
 
-void ghid_gl_draw_grid_local(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t cy)
+void ghid_gl_draw_grid_local(rnd_design_t *hidlib, rnd_coord_t cx, rnd_coord_t cy)
 {
 	if (hidlib->grid <= 0)
 		return;
@@ -116,10 +116,10 @@ void ghid_gl_draw_grid_local(rnd_hidlib_t *hidlib, rnd_coord_t cx, rnd_coord_t c
 	/* cx and cy are the actual cursor snapped to wherever - round them to the nearest real grid point */
 	grid_local_x = (cx / hidlib->grid) * hidlib->grid + hidlib->grid_ox;
 	grid_local_y = (cy / hidlib->grid) * hidlib->grid + hidlib->grid_oy;
-	grid_local_radius = rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.radius;
+	grid_local_radius = rnd_conf.editor.local_grid.radius;
 }
 
-static void ghid_gl_draw_grid(rnd_hidlib_t *hidlib, rnd_box_t *drawn_area)
+static void ghid_gl_draw_grid(rnd_design_t *hidlib, rnd_box_t *drawn_area)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
 	rnd_coord_t grd = hidlib->grid;
@@ -128,31 +128,31 @@ static void ghid_gl_draw_grid(rnd_hidlib_t *hidlib, rnd_box_t *drawn_area)
 		grd = 1;
 
 	if ((Vz(hidlib->grid) < RND_MIN_GRID_DISTANCE) || (!rnd_conf.editor.draw_grid)) {
-		if (!rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.sparse)
+		if (!rnd_conf.editor.global_grid.sparse)
 			return;
-		grd *= (rnd_gtk_conf_hid.plugins.hid_gtk.global_grid.min_dist_px / Vz(grd));
+		grd *= (rnd_conf.editor.global_grid.min_dist_px / Vz(grd));
 	}
 
 	hidgl_set_grid_color(priv->grid_color.fr, priv->grid_color.fg, priv->grid_color.fb);
 
-	if (rnd_gtk_conf_hid.plugins.hid_gtk.local_grid.enable)
+	if (rnd_conf.editor.local_grid.enable)
 		hidgl_draw_local_grid(hidlib, grd, grid_local_x, grid_local_y, grid_local_radius, ghidgui->port.view.coord_per_px, rnd_conf.editor.cross_grid);
 	else
 		hidgl_draw_grid(hidlib, grd, drawn_area, ghidgui->port.view.coord_per_px, rnd_conf.editor.cross_grid);
 }
 
-static void rnd_gl_draw_texture(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t bw, rnd_coord_t bh)
+static void rnd_gl_draw_texture(rnd_design_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t bw, rnd_coord_t bh)
 {
 	hidgl_draw_texture_rect(ox, oy, ox+bw, oy+bh, gpm->cache.lng);
 }
 
-static void ghid_gl_uninit_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm)
+static void ghid_gl_uninit_pixmap(rnd_design_t *hidlib, rnd_gtk_pixmap_t *gpm)
 {
 	if (gpm->cache.lng != 0)
 		hidgl_texture_free(gpm->cache.lng);
 }
 
-static void ghid_gl_draw_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t bw, rnd_coord_t bh)
+static void ghid_gl_draw_pixmap(rnd_design_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd_coord_t ox, rnd_coord_t oy, rnd_coord_t bw, rnd_coord_t bh)
 {
 	if (gpm->cache.lng == 0) {
 		int width = gpm->pxm->sx;
@@ -175,10 +175,10 @@ static void ghid_gl_draw_pixmap(rnd_hidlib_t *hidlib, rnd_gtk_pixmap_t *gpm, rnd
 	rnd_gl_draw_texture(hidlib, gpm, ox, oy, bw, bh);
 }
 
-static void ghid_gl_draw_bg_image(rnd_hidlib_t *hidlib)
+static void ghid_gl_draw_bg_image(rnd_design_t *hidlib)
 {
 	if (ghidgui->bg_pixmap.image != NULL)
-		ghid_gl_draw_pixmap(hidlib, &ghidgui->bg_pixmap, 0, 0, hidlib->size_x, hidlib->size_y);
+		ghid_gl_draw_pixmap(hidlib, &ghidgui->bg_pixmap, 0, 0, hidlib->dwg.X2, hidlib->dwg.Y2);
 }
 
 /* Config helper functions for when the user changes color preferences.
@@ -198,7 +198,14 @@ void ghid_gl_set_special_colors(rnd_conf_native_t *cfg)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
 
-	if (((RND_CFT_COLOR *) cfg->val.color == &rnd_conf.appearance.color.background)) {
+	if (cfg == NULL) {
+		/* update all from current conf */
+		priv->bg_color = rnd_conf.appearance.color.background;
+		priv->offlimits_color = rnd_conf.appearance.color.off_limit;
+		priv->grid_color = rnd_conf.appearance.color.grid;
+		set_special_grid_color();
+	}
+	else if (((RND_CFT_COLOR *) cfg->val.color == &rnd_conf.appearance.color.background)) {
 		priv->bg_color = cfg->val.color[0];
 	}
 	else if ((RND_CFT_COLOR *)cfg->val.color == &rnd_conf.appearance.color.off_limit) {
@@ -374,7 +381,7 @@ static void ghid_gl_notify_mark_change(rnd_hid_t *hid, rnd_bool changes_complete
 	ghid_gl_invalidate_all(hid);
 }
 
-static void ghid_gl_show_crosshair(rnd_hidlib_t *hidlib, gboolean paint_new_location, rnd_coord_t minx, rnd_coord_t miny, rnd_coord_t maxx, rnd_coord_t maxy)
+static void ghid_gl_show_crosshair(rnd_design_t *hidlib, gboolean paint_new_location, rnd_coord_t minx, rnd_coord_t miny, rnd_coord_t maxx, rnd_coord_t maxy)
 {
 	static int done_once = 0;
 	static rnd_gtk_color_t cross_color;
@@ -420,28 +427,37 @@ static void ghid_gl_screen_update(void)
 {
 }
 
-/* Settles background color + inital GL configuration, to allow further drawing in GL area.
-    (w, h) describes the total area concerned, while (xr, yr, wr, hr) describes area requested by an expose event.
-    The color structure holds the wanted solid back-ground color, used to first paint the exposed drawing area. */
+/* Settles background color + inital GL configuration, to allow further
+   drawing in GL area. (w, h) describes the total area concerned,
+   while (xr, yr, wr, hr) describes area requested by an expose event.
+   The color structure holds the wanted solid back-ground color, used to
+   first paint the exposed drawing area. */
 static void rnd_gl_draw_expose_init(int w, int h, int xr, int yr, int wr, int hr, rnd_color_t *bg_c)
 {
 	hidgl_stencil_init();
-	hidgl_expose_init(w, h, bg_c);
+	hidgl_expose_init(0, 0, w, h, bg_c);
 }
 
 static gboolean ghid_gl_drawing_area_expose_cb_common(rnd_hid_t *hid, GtkWidget *widget, rnd_gtk_expose_t *ev, void *vport)
 {
 	rnd_gtk_port_t *port = vport;
-	rnd_hidlib_t *hidlib = ghidgui->hidlib;
+	rnd_design_t *hidlib = ghidgui->hidlib;
 	render_priv_t *priv = port->render_priv;
 	GtkAllocation allocation;
 	rnd_hid_expose_ctx_t ctx;
 	double tx, ty, zx, zy, zz;
+	rnd_hid_t *hid_save;
+
+	/* save rnd_render and restore at the end: we may get an async expose draw
+	   while it is set to an exporter */
+	hid_save = rnd_render;
+	rnd_render = hid;
 
 	gtkc_widget_get_allocation(widget, &allocation);
 
 	ghid_gl_start_drawing(port);
 
+	ctx.design = hidlib;
 	ctx.view.X1 = MIN(Px(0), Px(allocation.width + 1));
 	ctx.view.Y1 = MIN(Py(0), Py(allocation.height + 1));
 	ctx.view.X2 = MAX(Px(0), Px(allocation.width + 1));
@@ -452,12 +468,12 @@ static gboolean ghid_gl_drawing_area_expose_cb_common(rnd_hid_t *hid, GtkWidget
 	zx = (rnd_conf.editor.view.flip_x ? -1. : 1.) / port->view.coord_per_px;
 	zy = (rnd_conf.editor.view.flip_y ? -1. : 1.) / port->view.coord_per_px;
 	zz = ((rnd_conf.editor.view.flip_x == rnd_conf.editor.view.flip_y) ? 1. : -1.) / port->view.coord_per_px;
-	tx = rnd_conf.editor.view.flip_x ? port->view.x0 - hidlib->size_x : -port->view.x0;
-	ty = rnd_conf.editor.view.flip_y ? port->view.y0 - hidlib->size_y : -port->view.y0;
+	tx = rnd_conf.editor.view.flip_x ? port->view.x0 - hidlib->dwg.X2 : -port->view.x0;
+	ty = rnd_conf.editor.view.flip_y ? port->view.y0 - hidlib->dwg.Y2 : -port->view.y0;
 	hidgl_set_view(tx, ty, zx, zy, zz);
 
 	/* Draw PCB background, before PCB primitives */
-	hidgl_draw_initial_fill(0, 0, hidlib->size_x, hidlib->size_y, priv->bg_color.fr, priv->bg_color.fg, priv->bg_color.fb);
+	hidgl_draw_initial_fill(hidlib->dwg.X1, hidlib->dwg.Y1, hidlib->dwg.X2, hidlib->dwg.Y2, priv->bg_color.fr, priv->bg_color.fg, priv->bg_color.fb);
 	ghid_gl_draw_bg_image(hidlib);
 
 	ghid_gl_invalidate_current_gc();
@@ -482,11 +498,13 @@ static gboolean ghid_gl_drawing_area_expose_cb_common(rnd_hid_t *hid, GtkWidget
 
 	ghid_gl_end_drawing(port);
 
+	rnd_render = hid_save;
+
 	return FALSE;
 }
 
 /* Assumes gl context is set up for drawing in the target widget */
-static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_hidlib_t *hidlib, rnd_gtk_expose_t *ev, rnd_hid_expose_t expcall, rnd_hid_expose_ctx_t *ctx, long widget_xs, long widget_ys)
+static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_design_t *hidlib, rnd_gtk_expose_t *ev, rnd_hid_preview_expose_t expcall, rnd_hid_expose_ctx_t *ctx, long widget_xs, long widget_ys)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
 	double xz, yz, vw, vh;
@@ -523,7 +541,7 @@ static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_hidlib_t *hidlib,
 
 	ghidgui->port.render_priv->in_context = rnd_true;
 
-	hidgl_expose_init(widget_xs, widget_ys, &priv->bg_color);
+	hidgl_expose_init(0, 0, widget_xs, widget_ys, &priv->bg_color);
 
 	/* call the drawing routine */
 	ghid_gl_invalidate_current_gc();
@@ -532,8 +550,8 @@ static void ghid_gl_preview_expose_common(rnd_hid_t *hid, rnd_hidlib_t *hidlib,
 	zx = (rnd_conf.editor.view.flip_x ? -1. : 1.) / ghidgui->port.view.coord_per_px;
 	zy = (rnd_conf.editor.view.flip_y ? -1. : 1.) / ghidgui->port.view.coord_per_px;
 	zz = 1;
-	tx = rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->size_x : -ghidgui->port.view.x0;
-	ty = rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->size_y : -ghidgui->port.view.y0;
+	tx = rnd_conf.editor.view.flip_x ? ghidgui->port.view.x0 - hidlib->dwg.X2 : -ghidgui->port.view.x0;
+	ty = rnd_conf.editor.view.flip_y ? ghidgui->port.view.y0 - hidlib->dwg.Y2 : -ghidgui->port.view.y0;
 	hidgl_set_view(tx, ty, zx, zy, zz);
 
 	rnd_gui->coord_per_pix = ghidgui->port.view.coord_per_px;
