@@ -21,17 +21,18 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
 #include "ui_zoompan.h"
 
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/hidlib.h>
+#include <librnd/plugins/lib_hid_common/lib_hid_common.h>
 #include "glue_common.h"
 
 double rnd_gtk_clamp_zoom(const rnd_gtk_view_t *vw, double coord_per_px)
@@ -43,7 +44,7 @@ double rnd_gtk_clamp_zoom(const rnd_gtk_view_t *vw, double coord_per_px)
 	/* max zoom is calculated so that zoom * canvas_size * 2 doesn't overflow rnd_coord_t */
 	max_zoom_w = (double)RND_COORD_MAX / (double)vw->canvas_width;
 	max_zoom_h = (double)RND_COORD_MAX / (double)vw->canvas_height;
-	max_zoom = MIN(max_zoom_w, max_zoom_h) / 2.0;
+	max_zoom = MAX(max_zoom_w, max_zoom_h);
 
 	out_zoom = coord_per_px;
 	if (out_zoom < min_zoom)
@@ -63,17 +64,29 @@ static void uiz_pan_common(rnd_gtk_view_t *v)
 	 * event so convert it back to event coordinates temporarily. */
 	rnd_gtk_coords_design2event(v, v->design_x, v->design_y, &event_x, &event_y);
 
-	/* Don't pan so far the board is completely off the screen */
-	v->x0 = MAX(-v->width, v->x0);
-	v->y0 = MAX(-v->height, v->y0);
 
+	/* Don't pan so far the design is completely off the screen */
 	if (v->use_max_hidlib) {
-		v->x0 = MIN(v->x0, VIEW_HIDLIB(v)->size_x);
-		v->y0 = MIN(v->y0, VIEW_HIDLIB(v)->size_y);
+		if (LOCALFLIPX(v))
+			v->x0 = MAX(-v->width, v->x0);
+		else
+			v->x0 = MAX(VIEW_HIDLIB(v)->dwg.X1 - v->width, v->x0);
+
+		if (LOCALFLIPY(v))
+			v->y0 = MAX(-v->height, v->y0);
+		else
+			v->y0 = MAX(VIEW_HIDLIB(v)->dwg.Y1 - v->height, v->y0);
+
+		v->x0 = MIN(v->x0, VIEW_HIDLIB(v)->dwg.X2 + (VIEW_HIDLIB(v)->dwg.X2 - VIEW_HIDLIB(v)->dwg.X1));
+		v->y0 = MIN(v->y0, VIEW_HIDLIB(v)->dwg.Y2 + (VIEW_HIDLIB(v)->dwg.Y2 - VIEW_HIDLIB(v)->dwg.Y1));
 	}
 	else {
 		assert(v->max_width > 0);
 		assert(v->max_height > 0);
+
+		v->x0 = MAX(-v->width, v->x0);
+		v->y0 = MAX(-v->height, v->y0);
+
 		v->x0 = MIN(v->x0, v->max_width);
 		v->y0 = MIN(v->y0, v->max_height);
 	}
@@ -103,18 +116,31 @@ rnd_bool rnd_gtk_coords_event2design(const rnd_gtk_view_t *v, int event_x, int e
 	return rnd_true;
 }
 
+void rnd_gtk_zoom_clamp_overflow(rnd_gtk_view_t *v)
+{
+	double maxx, maxy;
+
+	maxx = v->coord_per_px * v->canvas_width;
+	if (maxx > RND_COORD_MAX)
+		v->coord_per_px = ((double)RND_COORD_MAX * 0.99) / (double)v->canvas_width;
+	maxy = v->coord_per_px * v->canvas_height;
+	if (maxy > RND_COORD_MAX)
+		v->coord_per_px = ((double)RND_COORD_MAX * 0.99) / (double)v->canvas_height;
+}
+
 void rnd_gtk_zoom_post(rnd_gtk_view_t *v)
 {
 	v->coord_per_px = rnd_gtk_clamp_zoom(v, v->coord_per_px);
+	rnd_gtk_zoom_clamp_overflow(v);
 	v->width = v->canvas_width * v->coord_per_px;
 	v->height = v->canvas_height * v->coord_per_px;
 }
 
 /* gport->view.coord_per_px:
  * zoom value is PCB units per screen pixel.  Larger numbers mean zooming
- * out - the largest value means you are looking at the whole board.
+ * out - the largest value means you are looking at the whole design.
  *
- * gport->view_width and gport->view_height are in PCB coordinates
+ * gport->view_width and gport->view_height are in librnd coordinates
  */
 void rnd_gtk_zoom_view_abs(rnd_gtk_view_t *v, rnd_coord_t center_x, rnd_coord_t center_y, double new_zoom)
 {
@@ -156,7 +182,7 @@ void rnd_gtk_zoom_view_rel(rnd_gtk_view_t *v, rnd_coord_t center_x, rnd_coord_t
 	rnd_gtk_zoom_view_abs(v, center_x, center_y, v->coord_per_px * factor);
 }
 
-/* Side-correct version - long term this will be kept and the other is removed */
+/* Side-correct version zoom-to-window */
 void rnd_gtk_zoom_view_win(rnd_gtk_view_t *v, rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2, int setch)
 {
 	double xf, yf;
