@@ -19,23 +19,24 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include <librnd/rnd_config.h>
 #include <librnd/core/actions.h>
 #include <librnd/core/event.h>
-#include <librnd/core/hid.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_unit.h>
-#include <librnd/core/hid_init.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_unit.h>
+#include <librnd/hid/hid_init.h>
 #include "dlg_export.h"
 
 typedef struct{
 	RND_DAD_DECL_NOINIT(dlg)
 	int active; /* already open - allow only one instance */
+	void *appspec;
 
 	int tabs, len;
 
@@ -68,7 +69,7 @@ static rnd_hid_attr_val_t *get_results(export_ctx_t *export_ctx, int id)
 static void export_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
 {
 	export_ctx_t *export_ctx = caller_data;
-	rnd_hidlib_t *hl = rnd_gui->get_dad_hidlib(hid_ctx);
+	rnd_design_t *hl = rnd_gui->get_dad_design(hid_ctx);
 	int h, wid;
 
 
@@ -80,7 +81,7 @@ static void export_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *att
 
 			rnd_render = export_ctx->hid[h];
 			rnd_event(hl, RND_EVENT_EXPORT_SESSION_BEGIN, NULL);
-			export_ctx->hid[h]->do_export(export_ctx->hid[h], results);
+			export_ctx->hid[h]->do_export(export_ctx->hid[h], hl, results, export_ctx->appspec);
 			rnd_event(hl, RND_EVENT_EXPORT_SESSION_END, NULL);
 			rnd_render = render_save;
 			free(results);
@@ -128,7 +129,7 @@ static void export_close_cb(void *caller_data, rnd_hid_attr_ev_t ev)
 	memset(ctx, 0, sizeof(export_ctx_t)); /* reset all states to the initial - includes ctx->active = 0; */
 }
 
-static void rnd_dlg_export(const char *title, int exporters, int printers)
+void rnd_dlg_export(const char *title, int exporters, int printers, rnd_design_t *dsg, void *appspec)
 {
 	rnd_hid_t **hids;
 	int n, i, *exp_attr;
@@ -154,6 +155,7 @@ static void rnd_dlg_export(const char *title, int exporters, int printers)
 	export_ctx.button = malloc(sizeof(int) * (export_ctx.len));
 	export_ctx.numo = malloc(sizeof(int) * (export_ctx.len));
 	export_ctx.aa = malloc(sizeof(rnd_hid_attr_val_t *) * (export_ctx.len));
+	export_ctx.appspec = appspec;
 
 	for(i = n = 0; hids[n] != NULL; n++) {
 		if (((exporters && hids[n]->exporter) || (printers && hids[n]->printer)) && (!hids[n]->hide_from_gui)) {
@@ -162,12 +164,17 @@ static void rnd_dlg_export(const char *title, int exporters, int printers)
 				continue;
 			}
 
+			/* skip exporters that can not export the current design with appspec */
+			if ((hids[n]->can_export != NULL) && !hids[n]->can_export(hids[n], dsg, appspec))
+				continue;
+
 			export_ctx.tab_name[i] = hids[n]->name;
 			export_ctx.hid[i] = hids[n];
 			i++;
 		}
 	}
 
+	export_ctx.len = i;
 	export_ctx.tab_name[i] = NULL;
 
 	RND_DAD_BEGIN_VBOX(export_ctx.dlg);
@@ -177,7 +184,7 @@ static void rnd_dlg_export(const char *title, int exporters, int printers)
 			export_ctx.tabs = RND_DAD_CURRENT(export_ctx.dlg);
 			for(n = 0; n < export_ctx.len; n++) {
 				int numo;
-				const rnd_export_opt_t *opts = export_ctx.hid[n]->get_export_options(export_ctx.hid[n], &numo);
+				const rnd_export_opt_t *opts = export_ctx.hid[n]->get_export_options(export_ctx.hid[n], &numo, dsg, appspec);
 				rnd_hid_attr_val_t *args = export_ctx.hid[n]->argument_array;
 				export_ctx.numo[n] = numo;
 				export_ctx.aa[n] = args;
@@ -250,17 +257,17 @@ static void rnd_dlg_export(const char *title, int exporters, int printers)
 
 const char rnd_acts_ExportDialog[] = "ExportDialog()\n";
 const char rnd_acth_ExportDialog[] = "Open the export dialog.";
-fgw_error_t rnd_act_ExportDialog(fgw_arg_t *ores, int oargc, fgw_arg_t *oargv)
+fgw_error_t rnd_act_ExportDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_dlg_export("Export to file", 1, 0);
+	rnd_dlg_export("Export to file", 1, 0, RND_ACT_DESIGN, NULL);
 	return 0;
 }
 
 const char rnd_acts_PrintDialog[] = "PrintDialog()\n";
 const char rnd_acth_PrintDialog[] = "Open the print dialog.";
-fgw_error_t rnd_act_PrintDialog(fgw_arg_t *ores, int oargc, fgw_arg_t *oargv)
+fgw_error_t rnd_act_PrintDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_dlg_export("Print", 0, 1);
+	rnd_dlg_export("Print", 0, 1, RND_ACT_DESIGN, NULL);
 	return 0;
 }
 
