@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -39,15 +39,15 @@
 #include <genvector/vti0.h>
 #include <genregex/regex_se.h>
 #include <librnd/core/error.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_tree.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_tree.h>
 #include <librnd/core/globalconst.h>
 #include <librnd/core/compat_fs.h>
 #include <librnd/core/safe_fs.h>
 #include <librnd/core/safe_fs_dir.h>
 #include <librnd/core/event.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 
 #include "dialogs_conf.h"
 #include "dlg_fileselect.h"
@@ -89,7 +89,7 @@ typedef struct{
 	char *cwd;
 	int cwd_offs[FSD_MAX_DIRS]; /* string lengths for each dir button within ->cwd */
 	vtde_t des;
-	rnd_hidlib_t *hidlib;
+	rnd_design_t *hidlib;
 	rnd_hid_fsd_flags_t flags;
 	const rnd_hid_fsd_filter_t *flt;
 	const char *history_tag;
@@ -533,7 +533,7 @@ static void edit_enter_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t
 	rnd_gui->attr_dlg_set_value(ctx->dlg_hid_ctx, ctx->wpath, &hv);
 }
 
-TODO("closing from within a tree table causes a gtk/glib segf");
+TODO("closing from within a tree table causes a gtk/glib segf")
 static void timed_close_cb(rnd_hidval_t user_data)
 {
 	static rnd_dad_retovr_t retovr;
@@ -848,9 +848,8 @@ char *rnd_dlg_fileselect(rnd_hid_t *hid, const char *title, const char *descr, c
 		RND_DAD_END(ctx->dlg);
 
 		/* lists */
-		RND_DAD_BEGIN_HPANE(ctx->dlg);
+		RND_DAD_BEGIN_HPANE(ctx->dlg, "left-right");
 			RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-			RND_DAD_SET_PANE_NAME(ctx->dlg, "left-right");
 
 			RND_DAD_BEGIN_VBOX(ctx->dlg); /* shortcuts */
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
@@ -1109,3 +1108,34 @@ fgw_error_t rnd_act_FsdTest(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	return -1;
 }
 
+const char rnd_acts_FsdSimple[] = "FsdSimple(title, descr, default_file, default_ext, history_tag, [read])";
+const char rnd_acth_FsdSimple[] = "File selection dialog, simplified API; for meaning of the arguments, see the HID API doc.";
+fgw_error_t rnd_act_FsdSimple(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+{
+	rnd_hid_fsd_flags_t flags = 0;
+	char *resfn;
+	const char *title, *descr, *default_file, *default_ext, *history_tag, *sflags = NULL;
+
+	RND_ACT_CONVARG(1, FGW_STR, FsdSimple, title = argv[1].val.cstr);
+	RND_ACT_CONVARG(2, FGW_STR, FsdSimple, descr = argv[2].val.cstr);
+	RND_ACT_CONVARG(3, FGW_STR, FsdSimple, default_file = argv[3].val.cstr);
+	RND_ACT_CONVARG(4, FGW_STR, FsdSimple, default_ext = argv[4].val.cstr);
+	RND_ACT_CONVARG(5, FGW_STR, FsdSimple, history_tag = argv[5].val.cstr);
+	RND_ACT_MAY_CONVARG(6, FGW_STR, FsdSimple, sflags = argv[6].val.cstr);
+
+	if (sflags != NULL) {
+		if (rnd_strcasecmp(sflags, "read") == 0) {
+			flags = RND_HID_FSD_READ;
+		}
+		else {
+			rnd_message(RND_MSG_ERROR, "FsdSimple: the last argument, flags, must be either \"read\" or empty\n");
+			return FGW_ERR_ARG_CONV;
+		}
+	}
+
+	resfn = rnd_dlg_fileselect(rnd_gui, title, descr, default_file, default_ext, NULL, history_tag, flags, NULL);
+
+	res->type = FGW_STR;
+	res->val.str = resfn;
+	return 0;
+}
