@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -30,7 +30,7 @@
 #include <librnd/core/plugins.h>
 #include <librnd/core/conf_hid.h>
 #include <librnd/core/event.h>
-#include <librnd/core/hid_menu.h>
+#include <librnd/hid/hid_menu.h>
 #include <librnd/core/actions.h>
 #include "grid_menu.h"
 #include "cli_history.h"
@@ -53,7 +53,7 @@
 
 conf_dialogs_t dialogs_conf;
 
-void rnd_hid_announce_gui_init(rnd_hidlib_t *hidlib)
+void rnd_hid_announce_gui_init(rnd_design_t *hidlib)
 {
 	rnd_hid_menu_merge_inhibit_inc();
 	rnd_event(hidlib, RND_EVENT_GUI_INIT, NULL);
@@ -65,7 +65,7 @@ static const char *lead_cookie = "lib_hid_common/user_lead";
 static const char *wplc_cookie = "lib_hid_common/window_placement";
 
 extern void rnd_dad_spin_update_global_coords(void);
-static void grid_unit_chg_ev(rnd_conf_native_t *cfg, int arr_idx)
+static void grid_unit_chg_ev(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	rnd_dad_spin_update_global_coords();
 }
@@ -92,6 +92,7 @@ rnd_action_t hid_common_action_list[] = {
 	{"Scroll", rnd_act_Scroll, rnd_acth_Scroll, rnd_acts_Scroll},
 	{"LogDialog", rnd_act_LogDialog, rnd_acth_LogDialog, rnd_acts_LogDialog},
 	{"FsdTest", rnd_act_FsdTest, rnd_acth_FsdTest, rnd_acts_FsdTest},
+	{"FsdSimple", rnd_act_FsdSimple, rnd_acth_FsdSimple, rnd_acts_FsdSimple},
 	{"Command", rnd_act_Command, rnd_acth_Command, rnd_acts_Command},
 	{"ManagePlugins", pcb_act_ManagePlugins, pcb_acth_ManagePlugins, pcb_acts_ManagePlugins},
 	{"ExportDialog", rnd_act_ExportDialog, rnd_acth_ExportDialog, rnd_acts_ExportDialog},
@@ -134,6 +135,8 @@ void pplg_uninit_lib_hid_common(void)
 	rnd_dlg_log_uninit();
 }
 
+extern void rnd_gui_act_init(void);
+
 int pplg_init_lib_hid_common(void)
 {
 	static rnd_conf_hid_callbacks_t ccb, ccbu;
@@ -173,5 +176,9 @@ int pplg_init_lib_hid_common(void)
 	if (nat != NULL)
 		rnd_conf_hid_set_cb(nat, conf_id, &ccbu);
 
+	rnd_hid_fileselect_imp = rnd_dlg_fileselect;
+
+	rnd_gui_act_init();
+
 	return 0;
 }
